#!/usr/bin/env python

#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
#       Attempt to test volume churn in a multi-node env. Attempts multi-node usage by creating/using 
#        a number of instances equal to the node count. 
#        Test Summary: 
#        
#        Usage Tests: 
#        - create 2 x nodes in zone volumes per zone
#        - launch instances to interact with ebs volumes per zone
#        - repeat for testcount -> attach volumes alternating nodes, then detach all volumes from all nodes
#        - terminate each instance and verify that any attached volumes return to available state
#       
#        Cleanup:
#        --if 'fof' flag is not set, will remove all volumes, instance, and snapshots created during this test
#
#    @author: clarkmatthew

import unittest
from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import EutesterTestResult
from ebstestsuite import EbsTestSuite
import argparse
import os



if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch

    testcase= EutesterTestCase(name='multi_node_churn')    
    testcase.setup_parser(description="Attempts to test and provide info on focused areas related to Eucalyptus EBS "
                                      "related functionality.", testlist=False)
    testcase.parser.add_argument('--count', type=int, help='Number of times to run attach/detach churn, default:10',
                                 default=10)
    testcase.parser.add_argument('--nodecount', type=int, help='Number of nodes in env, default:2',default=2)
    testcase.parser.add_argument('--fof', dest='fof', help="Freeze test on fail, do not remove or tear down test items",
                                 action='store_true', default=None)
    testcase.get_args()
    count = testcase.args.count
    fof =  testcase.args.fof if testcase.args.fof is not None else False
    nodecount = testcase.args.nodecount
    ebstestsuite= testcase.do_with_args(EbsTestSuite)
    if testcase.args.fof:
        testcase.clean_method = lambda: testcase.status("Freeze on fail flag is set, not cleaning!")
    else:
        testcase.clean_method = ebstestsuite.clean_created_resources
    testlist = ebstestsuite.test_multi_node(run=False, 
                                            count=int(count),
                                            nodecount=int(nodecount))
    ret = testcase.run_test_case_list(testlist)
    print "mutli node test exiting:("+str(ret)+")"
    exit(ret)

    
  



#
# Description:  Looks for a set of six images, if found attempts to install the images on the clc. 
#               If image is not found error is displayed.
#

import time
from eucaops import Eucaops
if __name__ == '__main__':
    clc_session = Eucaops( config_file="../input/2b_tested.lst", password="foobar", hostname="clc" )
    
    imglist = clc_session.sys("export EUSTORE_URL=http://192.168.51.187/eustore/ && eustore-describe-images")
    if imglist[0].find("euca-centos5.3-x86_64")>-1 and imglist[0].find("CentOS 5.3 1.3GB root"):
        clc_session.test_name("found centos 5.3 64 bit image")
    else:
        clc_session.fail("Output of eustore-describe-images doesn't match expected")

    if imglist[1].find("OS:centos") and imglist[1].find("Arch:x86_64") and imglist[1].find("Vers:old"):
        clc_session.test_name("found centos 5.3 64 bit image details")
    else:
        clc_session.fail("Output of eustore-describe-images doesn't match expected")

    if imglist[2].find("euca-debian5-x86_64")>-1 and imglist[2].find("Debian 5 1.3GB root"):
        clc_session.test_name("found debian 5 64 bit image")
    else:
        clc_session.fail("Output of eustore-describe-images doesn't match expected")

    if imglist[3].find("OS:debian") and imglist[3].find("Arch:x86_64") and imglist[3].find("Vers:old"):
        clc_session.test_name("found debian 5 64 bit image details")
    else:
        clc_session.fail("Output of eustore-describe-images doesn't match expected")

    if imglist[4].find("euca-ubuntu-9.04-x86_64")>-1 and imglist[4].find("Ubuntu 9.04 1.3GB root"):
        clc_session.test_name("found ubuntu 9.04 64 bit image")
    else:
        clc_session.fail("Output of eustore-describe-images doesn't match expected")

    if imglist[5].find("OS:centos") and imglist[5].find("Arch:x86_64") and imglist[5].find("Vers:old"):
        clc_session.test_name("found ubuntu 9.04 64 bit image details")
    else:
        clc_session.fail("Output of eustore-describe-images doesn't match expected")

    installret = clc_session.sys(cmd="export EUSTORE_URL=http://192.168.51.187/eustore/ && eustore-install-image -i euca-ubuntu-9.04-x86_64 -b eutest -k xen -d /disk1/storage", timeout=600)
    emi = installret[len(installret)-1]  #last line
    emi = emi[emi.find("emi-"):].strip()
    clc_session.test_name("installed image! " + emi)
    emilist = None;
    for i in [1, 2, 3, 4, 5]:
        emilist = clc_session.ec2.get_all_images(image_ids=[emi])
        if len(emilist):
            clc_session.test_name("emi " + emi + " was found via describe-images!")
            break
        else:
            print clc_session.ec2.get_all_images()
            time.sleep(10)
            if i == 5:
                clc_session.fail("eustore-install-image didn't properly install the image!")

    res = clc_session.run_instance(image=emilist[0])
    clc_session.terminate_instances(res)
    clc_session.sys(cmd="rm -rf /disk1/storage/*")
    clc_session.do_exit()



from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eutester.euinstance import EuInstance
import time






class Check_vol_on_stop_start(EutesterTestCase):
        # init with argument list that makes these mandatory, but can fetch them from special 'args' later. 
        def __init__(self):
            self.setuptestcase()
            self.setup_parser(description="Test EBS on BFEBS during restart", testlist=False)
            self.parser.add_argument('--count',
                                     help='Number of times to start/stop instance', default=5)
            self.parser.add_argument('--pause',
                                     help='Number of seconds to pause between tests',default=5)
            self.parser.add_argument('--size',
                                     help='Size of volume to create', default=1)
            self.get_args()
            self.set_arg('azone', self.args.zone)
            try:
                self.do_with_args(self.checkargs)
            except Exception:
                raise Exception("Mandotory arguments missing: emi and zone and (credpath or (config and password))")
            self.tester = self.do_with_args(Eucaops)
            self.volume = None
            self.instance = EuInstance()
            #self.tester = Eucaops()
            
        def checkargs(self,emi,zone, credpath=None, config=None, password=None):
            if emi and zone and (credpath or (config and password)):
                return
            raise Exception("Mandotory arguments missing: emi and zone and (credpath or (config and password))")
        
        def get_keypair(self,keypair=None):
            self.keypair = None
            if keypair:
                self.keypair=self.tester.add_keypair(keypair)
        
    
            
        def run_instance(self):
            image = self.do_with_args(self.tester.get_emi)
            reservation = self.do_with_args(self.tester.run_instance, image=image, username='root')
            self.instance = reservation.instances[0]
            
        def create_volume(self):
            self.volume = self.do_with_args(self.tester.create_volume)
        
        def attach_volume_to_instance(self):
            self.instance.attach_volume(self.volume)
        
        def stop_start_check(self,pause=5):
            self.instance.stop_instance_and_verify()
            self.volume.update()
            self.instance.start_instance_and_verify(checkvolstatus=True)
            time.sleep(pause)
        
            
        def run_list(self):
            list=[]
            list.append(self.create_testunit_from_method(self.run_instance, eof=True))
            list.append(self.create_testunit_from_method(self.create_volume, eof=True))
            list.append(self.create_testunit_from_method(self.attach_volume_to_instance, eof=True))
            for x in xrange(1,int(self.args.count)):
                list.append(self.create_testunit_by_name('stop_start_check'))
            self.run_test_case_list(list)
            
if __name__ == "__main__":
    test = Check_vol_on_stop_start()
    test.run_list()





# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
'''
Created on May 11, 2012
@author: clarkmatthew

Simple utility to read a given config file and parse out the configuration. 
Uses python's Config Parser but includes some utils to support legacy qa config files,
and file operations on local and remote machines containing keypair values.
To use this utility, the config file given must use the following format:
[prefix]key=value

example:
Add the following lines to a file called /tmp/config.txt
Note: 
This example is using the legacy config with ip, distro, and component info. 

>cat /tmp/config.txt
    1.1.1.1   CENTOS  5.7     64      REPO [CLC WS]
    1.1.1.2   CENTOS  5.7     64      REPO [NC00]
    
    [mytest]
    volumes=2
    image=centos.img
    
    [joestest]
    image=ubuntu.img
    name=joe
    iterations=3
    output=/tmp/joesoutput.txt


To retrieve config info from this file from within a test:
>conf = EuConfig(filename='/tmp/config.txt')
>myimage = conf.get_config('mytest','image')
>joesimage = conf.get('joestest','image')
>
>print myimage
 centos.img
 
>print conf.legacybuf
 1.1.1.1   CENTOS  5.7     64      REPO [CLC WS]
 1.1.1.2   CENTOS  5.7     64      REPO [NC00]


'''
import re
import io
import os
import ConfigParser
import time
import hashlib



class File_Util():
    #Possible file edit actions
    ADDTOLINE = 'ADDTOLINE'
    SWAP = 'SWAP'
    ADD = 'ADD'
    REMOVE = 'REMOVE'

    def __init__(self,
                 filepath,
                 ssh=None,
                 remove_blank_lines=True,
                 verbose=False):
        self.filepath = filepath
        self.lines = []
        self.ssh = ssh
        self.verbose = verbose
        self.remove_blank_lines = remove_blank_lines
        self.md5sum = None
        self.update()

    def file_open(self, filepath, read=True, write=False, create=True, flags=None):
        flags = flags or ""
        if not flags:
            if read:
                flags += 'r'
            if write:
                flags += 'w'
                if create:
                    flags += "+"
        if self.ssh:
            if not self.ssh.sftp:
                self.ssh.open_sftp()
            fileobj= self.ssh.sftp.file(filepath,flags)
        else:
            fileobj = open(filepath,flags)
        return fileobj


    def file_replace(self, from_path, to_path):
        if self.ssh:
            self.ssh.sys('mv ' + str(from_path) + " " + str(to_path))
        else:
            os.remove(to_path)
            os.rename(from_path, to_path)

    def get_md5(self, blocksize=65536):
        my_file = self.file_open(self.filepath, flags='rb')
        md5 = hashlib.md5()
        buf = my_file.read(blocksize)
        while len(buf) > 0:
            md5.update(buf)
            buf = my_file.read(blocksize)
        my_file.close()
        self.md5sum = md5.hexdigest()
        return md5.hexdigest()


    def has_changed(self):
        if self.md5sum != self.get_md5():
            return True
        else:
            return False

    def sanitize(self,dirty_string):
        clean_string = ""
        for c in dirty_string:
            if re.match('\W',c):
                clean_string += '\\' + c
            else:
                clean_string += c
        return clean_string

    def print_file(self, printmethod=None):
        for line in self.lines:
            if printmethod:
                printmethod(line.strip())
            else:
                print line.strip()

    def search_in_lines(self, search_pattern, after_pattern=None):
        if not after_pattern:
            found_after = True
        else:
            found_after = False
        for line in self.lines:
            if not found_after:
                if re.search(after_pattern, line):
                    found_after = True
            if found_after and re.search(search_pattern, line):
                return line
        return None

    def match_in_lines(self, match_pattern, after_pattern):
        if not after_pattern:
            found_after = True
        else:
            found_after = False
        for line in self.lines:
            if not found_after:
                if re.match(after_pattern, line):
                    found_after = True
            if found_after and re.match(match_pattern, line):
                return line
        return None

    def update(self, retries=3):
        while retries:
            retries -= 1
            self.lines = self.get_file_lines()
            if self.lines:
                self.md5sum = self.get_md5()
                return
            time.sleep(2)
        print 'No lines gathered in update'


    def get_file_lines(self):
        lines = []
        cfile = None
        try:
            cfile = self.file_open(self.filepath,read=True)
            cfile.seek(0)
            for line in cfile.readlines():
                if line.strip() or not self.remove_blank_lines:
                    lines.append(line)
        except Exception, e:
            if cfile:
                cfile.close()
            raise Exception('Error in get_file_lines: ' + str(e))
        finally:
            if cfile:
                cfile.close()
        return lines

    def file_edit_line(self,
                    new_line,
                    action = None,
                    search_pattern=None,
                    after_pattern=None,
                    single_action=True,
                    tempfilepath=None):
        if self.verbose:
            print "Starting file_edit_line..."
        tempfilepath = tempfilepath or str(self.filepath)+str('.tmp')
        tempfile = self.file_open(tempfilepath, write=True, create=True)
        self.update()
        updated = False
        if not action:
            raise Exception('file_edit_line needs File_Util action provided')
        action = str(action).upper()
        if not hasattr(self, action):
            raise Exception('Action:' + str(action) + ' not found as valid action?')
        if not after_pattern:
            start_write = True
        else:
            start_write = False
        #If no search criteria is specified, just write the line to the file...
        if start_write and not search_pattern:
            myfile = open(self.filepath, 'w')
            myfile.write(new_line)
            myfile.close()
            print new_line
        #Iterate through file, when search patterns have been satisfied, write new_line...
        try:
            for line in self.lines:
                if updated and single_action:
                    tempfile.write(line)
                    continue
                line = line.strip()
                if not start_write:
                    if re.search(after_pattern, line):
                        start_write = True
                if start_write:
                    if not search_pattern or re.search(search_pattern, line):
                        print "Found search pattern:" + str(search_pattern)
                        if action == File_Util.ADD:
                            if self.verbose: print "OLD:" + str(line)
                            line = str(line) + "\n" + new_line
                            if self.verbose: print "NEW:" + str(line)
                            updated = True
                        elif search_pattern and action == File_Util.ADDTOLINE:
                            if self.verbose: print "OLD:" + str(line)
                            line = str(line) + new_line
                            if self.verbose: print "NEW:" + str(line)
                            updated = True
                        elif search_pattern and action == File_Util.SWAP:
                            if self.verbose: print "OLD:" + str(line)
                            line = new_line
                            if self.verbose: print "NEW:" + str(line)
                            updated = True
                        elif search_pattern and action == File_Util.REMOVE:
                            if self.verbose: print "OLD:" + str(line)
                            if self.verbose: print "NEW:"
                            updated = True
                            continue
                if self.verbose: print "writing line:" +str(line)
                tempfile.write(line + "\n")
            if updated:
                self.file_replace(tempfilepath,self.filepath)
        except Exception, e:
            print "Error when editing file:" + str(e)
        finally:
            tempfile.close()
            if updated:
                self.update()



    def swap_existing_line(self, search_pattern, new_line, after_pattern=None ):
        return self.file_edit_line(new_line=new_line,
                                   search_pattern=search_pattern,
                                   after_pattern=after_pattern,
                                   action=File_Util.SWAP)

    def add_new_line(self, new_line, after_pattern=None, over_write=False):
        return self.file_edit_line(new_line=new_line,
                                   after_pattern=after_pattern,
                                   action=File_Util.ADD)
    def stat_file(self):
        if self.ssh:
            return self.ssh.sftp.stat(self.filepath)
        else:
            return os.stat(self.filepath)

    def add_to_existing_line(self, search_pattern,values_to_add, after_pattern=None):
        return self.file_edit_line(new_line=values_to_add,
                                   search_pattern=search_pattern,
                                   after_pattern=after_pattern,
                                   action=File_Util.ADDTOLINE)

    def remove_existing_line(self, search_pattern,after_pattern=None):
        return self.file_edit_line(new_line='',
                                   search_pattern=search_pattern,
                                   after_pattern=after_pattern,
                                   action= File_Util.REMOVE)



class Config_Item():
    def __init__(self, name, value, config_section):
        self.name = name
        self.value = value
        self.search_pattern = str(self.name) + "\s*="
        self.config_section = config_section

    def update(self):
        self.config_section = self.config_section.update()


    def config_file_set_this_line(self,value, quoted=True):
        new_value = str(value)
        if quoted:
            new_value = '"' + new_value.strip('"') + '"'
        new_value = str(self.name) + "=" + str(new_value)
        ret = self.config_section.config_file_swap_line(self.search_pattern, new_line=new_value)
        self.update()
        return ret

    def config_file_add_to_this_line(self, value_to_add):
        ret = self.config_section.config_file_add_to_existing_line(search_pattern=self.search_pattern, value_to_add=value_to_add)
        self.update()
        return ret

    def config_file_remove_this_line(self):
        ret = self.config_section.config_file_remove_line(search_pattern=self.search_pattern)
        self.update()
        return ret

    def __str__(self):
        return "SECTION:" + str(self.config_section.name) + ", VALUE:" + str(self.value)



class Config_Section():
    def __init__(self, name, config_manager, strip='"'):
        self.name = name
        self.config_manager = config_manager
        self.strip = strip
        self.file_util = self.config_manager.file_util
        self.section_pattern = self.get_section_pattern()
        self.update_attributes_for_section()


    def update(self):
        if self.config_manager.file_util.has_changed():
            self.config_manager.update()
            self.update_attributes_for_section()
        return self

    def get_item(self, name):
        item = None
        if hasattr(self, name):
            item = getattr(self, name)
        return item

    def get_section_pattern(self):
        if self.config_manager.has_a_section():
            section_pattern = '\[\s*' + str(self.name) + '\s*\]'
        else:
            section_pattern = None
        return section_pattern

    def update_attributes_for_section(self):
        existing_items = self.get_all_items()
        for item in self.config_manager.config.items(self.name):
            item_name = item[0]
            existing_item = self.get_item(name=item_name)
            if self.strip:
                value =  str(item[1]).strip(self.strip)
            else:
                value = item[1]
            self.add_item_to_section(item_name,value)
            if existing_item:
                existing_items.remove(existing_item)
        if existing_items:
            self.remove_all_items(items=existing_items)

    def write_new_item_to_section_in_config_file(self,key, value):
        new_line = str(key) + "=" + '"' + str(value).strip('"') + '"'
        self.config_file_add_new_line_to_section(new_line=new_line)

    def add_item_to_section(self, config_item_name, value):
        if hasattr(self, config_item_name):
            item = getattr(self, config_item_name)
            if not isinstance(item, Config_Item):
                raise Exception('Conflicting attribute name:' + str(config_item_name))
            item.__init__(config_item_name, value, self)
        else:
            new_item = Config_Item(config_item_name, value, self)
            setattr(self, config_item_name, new_item)

    def config_file_add_to_existing_line(self, search_pattern, value_to_add):
        return self.file_util.add_to_existing_line(search_pattern=search_pattern,
                                            value_to_add=value_to_add,
                                            after_pattern=self.section_string)


    def config_file_swap_line(self, search_pattern, new_line):
        return self.file_util.swap_existing_line(search_pattern=search_pattern,
                                          new_line=new_line,
                                          after_pattern=self.section_pattern)

    def config_file_remove_line(self, search_pattern):
        return self.file_util.remove_existing_line(search_pattern=search_pattern, after_pattern=self.section_pattern)

    def config_file_add_new_line_to_section(self, new_line, over_write=True):
        return self.file_util.add_new_line(new_line=new_line,
                                    after_pattern=self.section_pattern,
                                    over_write=over_write)


    def get_all_items(self, item_name=None):
        items = []
        for attr in self.__dict__:
            if isinstance(self.__dict__[attr], Config_Item):
                if not item_name or (item_name == attr):
                    items.append(self.__dict__[attr])
        return items

    def get_item(self,name):
        item = None
        items = self.get_all_items(item_name=name)
        if item:
            item = item[0]
        return item

    def remove_all_items(self, item_name=None, items=None):
        items = items or self.get_all_items(item_name=item_name)
        for item in items:
            delattr(self, item.name)


class EuConfig():
    
    def __init__(self,
                 filename='../input/2btested.lst',
                 file_util=None,
                 ssh=None,
                 config_lines=None,
                 debugmethod=None,
                 verbose=True,
                 default_section_name='DEFAULTS',
                 auto_detect_memo_section=True,
                 make_section_attrs=True,
                 legacy_qa_config=False,
                 preserve_option_case=True,
                 strip_values = '"'):
        self.config = None
        self.preserve_option_case = preserve_option_case
        self.legacy_qa_config = legacy_qa_config
        self.file_util = file_util
        self.ssh = ssh
        self.auto_detect_memo_section=auto_detect_memo_section
        self.debugmethod = debugmethod
        self.make_section_attrs = make_section_attrs
        self.strip_values = strip_values
        self.filename = filename
        self.default_section_name = default_section_name or os.path.basename(self.filename).replace('.','_')
        self.verbose = verbose

        if not self.file_util:
            self.file_util = self.create_file_util_from_file(filepath = filename, ssh=ssh, verbose=False)

        #read the file into a list of lines
        self.lines = config_lines or self.file_util.lines
        self.config = None
        if self.auto_detect_memo_section and self.has_legacy_memo_section_marker(lines=self.lines):
            self.legacy_qa_config=True
        self.update()



    def check_and_add_default_section(self, default_section_name=None):
        default_section_name = default_section_name or self.default_section_name
        if self.has_a_section():
            return
        default_section_name = default_section_name or self.default_section_name
        default_section = '[' + str(default_section_name) + ']\n'
        self.lines.insert(0, default_section)

    def has_legacy_memo_section_marker(self, lines=None):
        lines = lines or self.lines
        for line in lines:
            if re.match("^MEMO\s*$", line):
                return True
        return False

    def has_a_section(self):
        for line in self.lines:
            if re.match("^\[\]+",  line):
                return True
        return False

    def update(self,lines=None):

        #read/re-read the file into a list of lines
        if lines:
            self.lines = lines
        else:
            self.file_util.update()
            self.lines = self.file_util.lines
            if not self.legacy_qa_config:
                self.check_and_add_default_section()

        #parse out any legacy config into a separate buffer (to support older test config formats)
        self.legacybuf = self.get_legacy_config()

        #parse out remaining non-legacy config into another buffer
        self.configbuf = self.get_config_buf()

        #create our configParser object using the config buffer
        self.config = None

        self.populate_config_parser_from_buf(buf=self.configbuf)


    @classmethod
    def create_file_util_from_file(cls, filepath, ssh=None, verbose=False):
        """
        Creates either a File_Util obj, working on either a local or remote file (if ssh is provided).
        :param filepath: string representing the local or remote file path
        :param ssh: eutester ssh_connection obj if file is remote
        :return: File_Util obj
        """
        file_util = File_Util(filepath=filepath, ssh=ssh)
        return file_util


    def add_new_item_to_config_file_under_section(self,section_name, config_item_name, value):
        self.debug('Attempting to add new config item:\n' + \
                    'config file:' +str(self.filename) + \
                    'section:' + str(section_name) + \
                    'config_item_name:' + str(config_item_name) + \
                    'value:' + str(value))
        
    def get(self,section,key):
        return self.config.get(section,key)


    def populate_config_parser_from_buf(self,
                                        buf=None,
                                        default_section_name=None,
                                        make_section_attrs=None,
                                        strip_values=None):
        #create our configParser object using the config buffer
        default_section_name = default_section_name or self.default_section_name
        make_section_attrs = make_section_attrs or self.make_section_attrs
        strip_values = strip_values or self.strip_values
        self.config = ConfigParser.RawConfigParser()
        if self.preserve_option_case:
            self.config.optionxform = str
        buf = buf or self.configbuf
        default_section_name=default_section_name or self.default_section_name
        try:
            self.config.readfp(io.BytesIO(buf))
        except ConfigParser.MissingSectionHeaderError, mshe:
            self.debug('Caught MissingSectionHeaderError')
            #A file with key pair format but no sections may have been fed in...?
            if default_section_name:
                buf = '[' + str(default_section_name) + ']\n' + str(buf)
                return self.create_config_parser_from_buf(buf=buf, create_default_section=False)
            else:
                raise mshe
        if make_section_attrs:
            self.make_sections(strip=strip_values)


        
    def debug(self, msg):
        if self.verbose:
            if self.debugmethod is not None:
                self.debugmethod(str(msg))
            else:
                print(str(msg))
        
        
    def get_legacy_config(self, lines=None):
        '''
        Gather all lines  from the given config until a section header is reached
        In order to be backwards compatible with previous configuration files
        will return a buffer of file read until the first section header
        '''
        buf=""
        if lines is None:
            lines = self.lines
        for line in lines:
            #read file until first section  header
            if re.match("^\[", line):
                #self.debug("Found section header, returning buf")
                return buf
            else:
                #filter out comments and blank lines
                if re.match("^\s+#",line) or re.match("^\s+$",line):
                    #self.debug("Ignoring legacy line:"+str(line))
                    continue
                else:
                    #self.debug("Adding line to legacy buf:"+str(line))
                    buf=buf+line
        return buf

    def uncomment_line(self, config_item):
        new_conf = []
        for line in self.lines:
            if re.match("^\s+#",line) and re.search(config_item,line):
                #self.debug("Ignoring legacy line:"+str(line))
                new_conf.append(line.strip('#'))
            else:
                new_conf.append(line)
        self.lines = new_conf
    
    def get_config_buf(self,lines=None, default_section_name=None):
        '''
        Gather config lines into list. 
        To support legacy config, will exclude all lines read prior to detecting a section header. 
        Will ignore blank lines and lines starting with "#" representing comments 
        '''
        buf=""
        start=False
        got_section = False
        if lines is None:
            lines = self.lines
        for line in lines:
            if re.match("^MEMO\s+", line):
                line = '['+str(line).strip()+']\n'
                got_section = True
            if re.match("^END_MEMO", line):
                continue
            #read file until first section  header
            if not start:
                if re.match("^\[", line):
                    #self.debug("Found fist section header, start reading in config buff")
                    got_section = True
                    buf=buf+line
                    start=True
            else:
                if re.match("^\s*#",line) or re.match("^\s+$",line) or not re.search("=",line):
                    #self.debug("Ignoring line for config buf:"+str(line))
                    continue
                else:
                    #self.debug("Adding line to config buff:"+str(line))
                    if not line.endswith('\n'):
                        line = line+"\n"
                    buf=buf+line

        return buf
                

    def make_sections(self, strip='"'):
        '''
        Creates section objects within this euconfig instance. These are dynamic for python console dev/debugging use,
        the use of these in testcases should be avoided.
        Example: The machine classes may have a file called eucalyptus_conf and create a section '[eucalyptus_conf]'
        This will add the section to self.config.sections, as well as create a Config_Section() object with attributes
        for each item which contain that item's value.
        For a machine called 'clc' values out of this config file can be access like this...
            >print clc.config.eucalyptus_conf.hypervisor
            > kvm

            >print clc.config.eucalyptus_conf.cloud_opts
            >-Debs.storage.manager=SANManager -Debs.san.provider=NetappProvider --debug
        '''
        if not self.config or not self.config.sections():
            self.debug('No config sections for:'+str(self.filename))
            return
        existing_sections = self.get_all_sections()
        #Check for any new sections, update any existing sections, and remove any that are no longer found
        for section in self.config.sections():
            self.debug('Creating/updating section: ' + str(section))
            existing_section = self.get_section(section)
            if existing_section:
                self.debug('Updating Existing Section: ' + str(section))
                existing_section.update_attributes_for_section()
                existing_sections.remove(existing_section)
            else:
                self.debug('Adding section:' + str(section))
                new_section = Config_Section(section,self, strip=strip)
                setattr(self, section, new_section)
        #Remove any pre-existing sections that no longer exist
        if existing_sections:
            self.remove_all_sections(sections=existing_sections)

    def get_all_sections(self, section_name=None):
        sections = []
        for attr in self.__dict__:
            if isinstance(self.__dict__[attr], Config_Section):
                if not section_name or (section_name == attr):
                    sections.append(self.__dict__[attr])
        return sections

    def get_section(self,name):
        section = None
        sections = self.get_all_sections(section_name=name)
        if sections:
            section = sections[0]
        return section

    def remove_all_sections(self, section_name=None, sections=None):
        sections = sections or self.get_all_sections(section_name=section_name)
        for section in sections:
            if not section_name or section.name == section_name:
                self.debug('Removing section:' +str(section.name))
                delattr(self, section.name)

    
        
            
        
        


#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

class ChangeProperty(EutesterTestCase):

    def __init__(self,extra_args = None):
        self.setuptestcase()
        self.setup_parser()
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.parser.add_argument("--property", default="cloud.euca_log_level")
        self.parser.add_argument("--value", default="DEBUG")
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config, password=self.args.password)

    def clean_method(self):
        pass

    def ChangeProperty(self):
        '''
        Change VLAN range to a given subset
        '''
        clc = self.tester.get_component_machines("clc")[0]
        clc.sys("source " + self.tester.credpath + "/eucarc && " + self.tester.eucapath + "/usr/sbin/euca-modify-property -p "+ self.args.property +"=" + self.args.value, code=0)

if __name__ == "__main__":
    testcase = ChangeProperty()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or ["ChangeProperty"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: Vic Iglesias vic.iglesias@eucalyptus.com
#
from eutester import Eutester
import boto
from boto.ec2.regioninfo import RegionInfo

class CFNops(Eutester):

    def __init__(self, endpoint=None, region=None, credpath=None, aws_access_key_id=None, aws_secret_access_key=None):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.user_id = None
        self.account_id = None
        super(CFNops, self).__init__(credpath=credpath)
        self.setup_cfn_connection(endpoint=endpoint, region=region, aws_access_key_id=self.aws_access_key_id, aws_secret_access_key=self.aws_secret_access_key)

    def setup_cfn_connection(self, endpoint=None, region=None, aws_access_key_id=None, aws_secret_access_key=None, path="/",port=443, is_secure=True, boto_debug=0):
        cfn_region = RegionInfo()
        if region:
            self.debug("Check region: " + str(region))
            try:
                if not endpoint:
                    cfn_region.endpoint = "cloudformation.{0}.amazonaws.com".format(region)
                else:
                    cfn_region.endpoint = endpoint
            except KeyError:
                raise Exception( 'Unknown region: %s' % region)
        else:
            cfn_region.name = 'eucalyptus'
            if endpoint:
                cfn_region.endpoint = endpoint
            else:
                cfn_region.endpoint = self.get_cfn_ip()

        try:
            cfn_connection_args = { 'aws_access_key_id' : aws_access_key_id,
                                    'aws_secret_access_key': aws_secret_access_key,
                                    'is_secure': is_secure,
                                    'debug':boto_debug,
                                    'port' : port,
                                    'path' : path,
                                    'region' : cfn_region}
            self.debug("Attempting to create cloudformation connection to " + self.get_cfn_ip() + ':' + str(port) + path)
            self.cloudformation = boto.connect_cloudformation(**cfn_connection_args)
        except Exception, e:
            self.critical("Was unable to create cloudformation connection because of exception: " + str(e))

    def get_cfn_ip(self):
        """Parse the eucarc for the AWS_CLOUDFORMATION_URL"""
        cfn_url = self.parse_eucarc("AWS_CLOUDFORMATION_URL")
        return cfn_url.split("/")[2].split(":")[0]

    def get_cfn_path(self):
        """Parse the eucarc for the AWS_CLOUDFORMATION_URL"""
        cfn_url = self.parse_eucarc("AWS_CLOUDFORMATION_URL")
        cfn_path = "/".join(cfn_url.split("/")[3:])
        return cfn_path

    def create_stack(self, stack_name, template_body, template_url=None, parameters=None):
        self.info("Creating stack: {0}".format(stack_name))
        self.cloudformation.create_stack(stack_name, template_body, template_url=template_url, parameters=parameters)

    def delete_stack(self, stack_name):
        self.info("Deleting stack: {0}".format(stack_name))
        self.cloudformation.delete_stack(stack_name)


#!/usr/bin/env python
#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#        Test Case is intended to test EBS services and persistance of EBS resource state(s) post EBS service
#        interruption. This should include volume and snapshots before and after service start, stop, restart, and SC
#        reboot.
#
#
#        Cleanup:
#        --if 'fof' flag is not set, will remove all volumes, instance, and snapshots created during this test
#
#    @author: clarkmatthew



from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import TestColor
from eucaops import ec2ops
#from eutester.euinstance import EuInstance
#from eutester.euvolume import EuVolume
#from eutester.eusnapshot import EuSnapshot
from eucaops import Eucaops
import eutester
import time
import copy

class Ebs_Persistance_Tests(EutesterTestCase):
    def __init__(self):
        #### Pre-conditions
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument('--testbfebs',
                                 help='If set will attempt test with a BFEBS image',
                                 action='store_true',
                                 default=False)
        self.parser.add_argument('--bfebsemi',
                                 help='BFEBS EMI to test with, otherwise will try to find one',
                                 default=None)
        self.parser.add_argument('--volsperinstance',
                                 type=int,
                                 help='Number of volumes to create per instance',
                                 default=2)
        self.parser.add_argument('--snapcount',
                                 type=int,
                                 help='Number of snapshots to create for snap related tests, default:2',
                                 default=2)
        self.parser.add_argument('--size',
                                 type=int,
                                 help='Size in GB for created volumes, default:1',
                                 default=1)
        self.parser.add_argument('--timepergig',
                                 type=int,
                                 help='Time allowed per gig size of volume during volume creation, default:120',
                                 default=300)
        self.parser.add_argument('--deletetimeout',
                                 type=int,
                                 help='Time allowed for volume to transition from deleting to deleted, default:120',
                                 default=120)
        self.parser.add_argument('--no_clean_on_exit',
                                 help='If set will not attempt to remove created test artifacts after running',
                                 action='store_true',
                                 default=False)

        self.get_args()
        # Setup basic eutester object
        self.tester = self.do_with_args(Eucaops)
        #replace default eutester debugger with eutestcase's for more verbosity...
        self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        self.reservation = None
        self.instance = None
        ### Add and authorize a group for the instance
        ### Generate a keypair for the instance
        if self.args.zone:
            self.zones = str(self.args.zone).split(',')
        else:
            self.zones = self.tester.get_zones()
        if not self.zones:
            raise Exception('No zones found for test?')

        self.debug('Using zones for testing: ' + ",".join(self.zones))
        self.groupname = 'jenkins'
        self.group = self.tester.add_group(self.groupname)
        self.tester.authorize_group(self.group)
        self.tester.authorize_group(self.group, protocol='icmp',port='-1')
        try:
            keys = self.tester.get_all_current_local_keys()
            if keys:
                self.keypair = keys[0]
            else:
                self.keypair = keypair = self.tester.add_keypair('qa214volumechurn'+str(time.time()))
        except Exception, ke:
            raise Exception("Failed to find/create a keypair, error:" + str(ke))

        ### Get an image to work with
        if self.args.emi:
            self.image = self.tester.get_emi(emi=str(self.args.emi))
        else:
            self.image = self.tester.get_emi(root_device_type="instance-store")
        if not self.image:
            raise Exception('couldnt find instance store image')
        self.clean_method = self.cleanup

        if self.args.testbfebs:
            if self.args.bfebsemi:
                self.bfebs_image = self.tester.get_emi(emi=str(self.args.emi))
                if self.image.root_device_type != "ebs":
                    raise Exception(str(self.args.bfebsemi) + ": Does not have EBS root_device_type")
            else:
                self.bfebs_image = self.tester.get_emi(root_device_type="ebs")
            if not self.image:
                raise Exception('"testbfebs" argument was set, but no BFEBS image found or provided')
        self.volumes = []
        self.instances = []
        self.bfebs_instances = []
        self.snapshots = []
        self.bfebs_volumes = []
        self.timepergig = self.args.timepergig
        self.clean_on_exit = True
        if self.args.no_clean_on_exit:
            self.clean_on_exit = False




    def cleanup(self, instances=True):
        '''
        Attempts to clean up test artifacts...
        '''
        try:
            self.tester.cleanup_artifacts()
        except Exception, e:
            tb = self.tester.get_traceback()
            raise Exception('Cleanupfailed:'+str(e) + "\n" +str(tb))

    def pretest1_pre_service_restart_launch_test_instances(self):
        """
        For each zone, check if there are multiple nodes. If so launch 2 instances per that zone (no need to do more for
        this test). If there is only 1 node launch 1 instance. This should produce a multi-node and multi-cluster
        test.This tests should be completed per zone prior to restarting services.
        """
        instances = []
        for zone in self.zones:
            count = 2 if len(self.tester.service_manager.get_all_node_controllers(part_name=str(zone))) > 1 else 1
            instances.extend(self.tester.run_image(image=self.image,
                                                   zone=zone,
                                                   min=count,
                                                   max=count,
                                                   group=self.group,
                                                   keypair=self.keypair,
                                                   monitor_to_running=False))
        self.instances = self.tester.monitor_euinstances_to_running(instances)


    def pretest2_pre_service_restart_create_volume_resources(self,
                                                     volsperinstance=2,
                                                     size=1,
                                                     timepergig=300):
        """
        Definition: Create volumes to be used in this test based upon volsperinstance and size args provided.
         This tests should be completed per zone prior to restarting services.

        :param volsperinstance: integer how many concurrent volumes to create/delete per instance
        :param size: integer size of volume(s) in GB
        :param timepergig: integer time allowed per GB in seconds during creation
        """
        volumes = []
        for zone in self.zones:
            instancecount = 0
            for instance in self.instances:
                if instance.placement == zone:
                    instancecount += 1
            volcount = instancecount*volsperinstance
            volumes.extend(self.tester.create_volumes(zone,
                                                      size=size,
                                                      count=volcount,
                                                      monitor_to_state=None,
                                                      timepergig=timepergig))
        self.volumes = self.tester.monitor_created_euvolumes_to_state(volumes,timepergig=timepergig)
        self.status('pre_service_restart_create_volume_resources, done',
                    testcolor=TestColor.get_canned_color('whiteonblue'))


    def pretest3_pre_service_restart_attach_all_volumes(self):
        '''
        Definition: Attach all volumes created in this test to all instances created in this test.
        This tests should be completed per zone prior to restarting services.
        '''
        self.status("\'pre_service_restart_attach_all_volumes\' starting...",
                    testcolor=TestColor.get_canned_color('whiteonblue'))

        for instance in self.instances:
            for x in xrange(0,self.args.volsperinstance):
                for vol in self.volumes:
                    if vol.zone == instance.placement and vol.status == 'available':
                        try:
                            instance.attach_volume(vol,timeout=90)
                        except ec2ops.VolumeStateException, vse:
                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                        testcolor=TestColor.get_canned_color('failred'))
                            time.sleep(10)
                            self.debug('Monitoring volume post VolumeStateException...')
                            vol.eutest_attached_status = None
                            self.tester.monitor_euvolumes_to_status([vol],status='in-use',attached_status='attached',timeout=60)
        self.status("\'pre_service_restart_attach_all_volumes\' done",
                        testcolor=TestColor.get_canned_color('whiteonblue'))


    def pretest4_pre_service_restart_create_snap_per_zone(self,timepergig=300):
        """
        Definition: Create a single snapshot. This tests should be completed per zone prior to restarting services.
        :param timepergig: integer time allowed per GB in seconds during creation
        :param deletetimeout: integer timeout in seconds waiting for volume to transition to 'deleted' state
        """
        snaps = []
        snapvols = []
        #Add one volume from each zone...
        for zone in self.zones:
            for vol in self.volumes:
                if vol.zone == zone:
                    snapvols.append(vol)
                    break
        #Create a snapshot of each volume...
        wait_on_progress = len(snapvols)*25
        for vol in snapvols:
            snaps.extend(self.tester.create_snapshots(vol,
                                                      count=1,
                                                      wait_on_progress=wait_on_progress,
                                                      monitor_to_completed=False,))
        self.tester.monitor_eusnaps_to_completed(snaps, wait_on_progress=wait_on_progress)
        self.snapshots = snaps


    def pretest5_pre_service_restart_launch_bfebs_test_instances(self):
        """
        For each zone, check if there are multiple nodes.
        If so launch 2 bfebs instances per that zone (no need to do more for
        this test). If there is only 1 node launch 1 instance. This should produce a multi-node and multi-cluster
        test. This tests should be completed per zone prior to restarting services.
        """
        instances = []
        for zone in self.zones:
            count = 2 if len(self.tester.service_manager.get_all_node_controllers(part_name=str(zone))) > 1 else 1
            instances.extend(self.tester.run_image(image=self.image,
                                                   zone=zone,
                                                   min=count,
                                                   max=count,
                                                   group=self.group,
                                                   keypair=self.keypair,
                                                   monitor_to_running=False))
        self.bfebs_instances = self.tester.monitor_euinstances_to_running(instances)


    def pretest6_pre_service_restart_create_bfebs_volume_resources(self,
                                                                    volsperinstance=2,
                                                                    size=1,
                                                                    timepergig=300):
        """
        Definition: Create volumes to be used in this test based upon volsperinstance and size args provided.
         This tests should be completed per zone prior to restarting services.

        :param volsperinstance: integer how many concurrent volumes to create/delete per instance
        :param size: integer size of volume(s) in GB
        :param timepergig: integer time allowed per GB in seconds during creation
        """
        volumes = []
        for zone in self.zones:
            instancecount = 0
            for instance in self.bfebs_instances:
                if instance.placement == zone:
                    instancecount += 1
            volcount = instancecount*volsperinstance
            volumes.extend(self.tester.create_volumes(zone,
                                                      size=size,
                                                      count=volcount,
                                                      monitor_to_state=None,
                                                      timepergig=timepergig))
        self.bfebs_volumes = self.tester.monitor_created_euvolumes_to_state(volumes,timepergig=timepergig)
        self.status('pretest6_pre_service_restart_create_bfebs_volume_resources, done',
                    testcolor=TestColor.get_canned_color('whiteonblue'))

    def pretest7_per_service_restart_attach_vols_to_bfebs_instances(self,volsperinstance=2):
        '''
        Definition: Attach all volumes created in this test for bfebs to all bfebs instances created in this test.
        This tests should be completed per zone prior to restarting services.
        '''
        self.status("\' pretest7_per_service_restart_attach_vols_to_bfebs_instances\' starting...",
                    testcolor=TestColor.get_canned_color('whiteonblue'))

        for instance in self.bfebs_instances:
            for x in xrange(0, volsperinstance):
                for vol in self.bfebs_volumes:
                    if vol.zone == instance.placement and vol.status == 'available':
                        try:
                            instance.attach_volume(vol,timeout=90)
                        except ec2ops.VolumeStateException, vse:
                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                        testcolor=TestColor.get_canned_color('failred'))
                            time.sleep(10)
                            self.debug('Monitoring volume post VolumeStateException...')
                            vol.eutest_attached_status = None
                            self.tester.monitor_euvolumes_to_status([vol],status='in-use',attached_status='attached',timeout=60)
        self.status("\' pretest7_per_service_restart_attach_vols_to_bfebs_instances\' done",
                    testcolor=TestColor.get_canned_color('whiteonblue'))

    def print_all_test_resources(self):
        self.status('Printing test resources prior to service interruption...',
                    testcolor=TestColor.get_canned_color('whiteonblue'))
        self.tester.print_euinstance_list(self.instances)
        self.tester.print_euvolume_list(self.volumes)
        self.tester.print_eusnapshot_list(self.snapshots)

    def reboot_sc_machine_verify_post_reboot(self, timeout = 600):
        """
        Definition: Restart the eucalyptus storage controller service
        """
        self.print_all_test_resources()
        #List to delay restart of storage controllers co-located with a cloud controller
        debug_str = ""
        #first make sure everything is good before we star the test...
        self.tester.service_manager.all_services_operational()
        storage_controllers = []
        self.zones = self.zones or self.tester.get_zones()
        for zone in self.zones:
            self.debug('Getting storage controllers for :' + str(zone))
            storage_controllers.extend(self.tester.service_manager.get_all_storage_controllers(partition=zone))
        self.tester.service_manager.print_services_list(storage_controllers)
        if not storage_controllers:
            raise Exception('Storage controller list was not populated for zones:' + ",".join(self.zones))
        for sc in storage_controllers:
            debug_str = ""
            all_services_on_sc = self.tester.service_manager.get_all_services_by_filter(hostname=sc.hostname)
            for service in all_services_on_sc:
                debug_str += "(" + str(service.hostname) + ":" + service.type + "), "
            self.status("Now rebooting machine hosting services:"+str(debug_str),
                        testcolor=TestColor.get_canned_color('whiteonblue'))
            sc.machine.reboot()
        if not storage_controllers:
            raise Exception('Storage controller list post reboot was not populated for zones:' + ",".join(self.zones))
        start = time.time()
        elapsed = 0
        waiting = copy.copy(storage_controllers)
        while elapsed < timeout and waiting:
            elapsed = int(time.time()-start)
            for sc in waiting:
                if self.tester.ping(sc.hostname, poll_count=1):
                    waiting.remove(sc)
            if waiting:
                debug_str = ""
                for sc in waiting:
                    debug_str += " " + str(sc.hostname) + ","
                self.debug("Waiting on SC's to become reachable post reboot:"
                           + str(debug_str) + ", elapsed:"+str(elapsed))
                time.sleep(10)
            else:
                self.status("All SC machines are reachable again, now wait for SSH...",
                            testcolor=TestColor.get_canned_color('whiteonblue'))
                break
        if waiting:
            raise Exception("SC machines were not reachable after: "+str(elapsed)+" seconds:"+str(debug_str))
        if not storage_controllers:
            raise Exception('Storage controller list was not populated after waiting for reachable')
        start = time.time()
        elapsed = 0
        waiting_for_ssh = copy.copy(storage_controllers)
        debug_str = ""
        while elapsed < 90 and waiting_for_ssh:
            elapsed = int(time.time()-start)
            for sc in waiting_for_ssh:
                try:
                    sc.machine.refresh_ssh()
                    sc.machine.cmd(" ")
                    waiting_for_ssh.remove(sc)
                except Exception, e:
                    self.debug('Failed to refresh ssh to:' + str(sc.hostname) + ', err:'+str(e))

            if waiting_for_ssh:
                debug_str = ""
                for sc in waiting:
                    debug_str += " " + str(sc.hostname) + ","
                self.debug("Waiting on SSH connections to SCs:"
                           + str(debug_str) + ", elapsed:"+str(elapsed))
                time.sleep(10)
            else:
                self.status("All SC machines have established SSH connections, now wait on services to come back...",
                            testcolor=TestColor.get_canned_color('whiteonblue'))
                break
        if waiting_for_ssh:
            raise("SC machines failed to establish SSH after: "+str(elapsed)+" seconds:"+str(debug_str))
        if not storage_controllers:
            raise Exception('Storage controller list was not populated after waiting for ssh')
        self.start_all_services_on_storage_controllers(storage_controllers)

    def start_tgtd_service(self,sc_list):
        for sc in sc_list:
            sc.machine.sys('service tgtd start', code=0)

    @eutester.Eutester.printinfo
    def start_all_services_on_storage_controllers(self, sc_list):
        self.status("Waiting for storage controller's services to start...",
                    testcolor=TestColor.get_canned_color('whiteonblue'))
        all_services_on_sc = []
        if not sc_list:
            raise Exception("sc_list was empty in: start_all_services_on_storage_controllers")
        #wait = 10 * 60
        for sc in sc_list:
            self.debug('Getting all services co-existing on sc: ' + str(sc.hostname))
            all_services_on_sc.extend(self.tester.service_manager.get_all_services_by_filter(hostname=sc.hostname))

        for service in all_services_on_sc:
            service.start()
        self.debug('Issued start to all services, now wait for: all_services_operational')
        self.tester.service_manager.all_services_operational()
        try:
            for service in all_services_on_sc:
                self.tester.service_manager.wait_for_service(service)
        except Exception, e:
            dbgout = self.tester.get_traceback()
            self.tester.service_manager.print_services_list(all_services_on_sc)
            dbgout += "\nFailed waiting for all services on rebooted machines to recover\n"
            raise Exception(dbgout + str(e))
        self.tester.service_manager.print_services_list(all_services_on_sc)


    def test1_post_service_interruption_check_attached_volumes(self, check_vols=None, check_instances=None):
        """
        Definition: Attempts to verify that volumes maintained their attached state through the SC reboot.
        """
        check_vols = check_vols or self.volumes
        check_instances = check_instances or self.instances
        write_length = 10000
        errmsg = ""
        self.tester.print_euvolume_list(check_vols)

        for vol in check_vols:
            vol.update()
            if vol.status == "in-use" and (vol.attach_data and vol.attach_data.status == 'attached' ):
                for instance in check_instances:
                    if instance.id == vol.attach_data.instance_id:
                        if not vol in instance.attached_vols:
                            errmsg += "Volume:" + str(vol.id) \
                                      + " is attached to " + str(instance.id) + ", but not in instance attached list?"
                        break
            else:
                errmsg += "Volume:" + str(vol.id) + ", status:" + str(vol.status) \
                          + " was not attached post service interruption \n"

        if errmsg:
            raise Exception(errmsg)

        self.status("Attached state passed. Now checking read/write with attached volumes...",
                    testcolor=TestColor.get_canned_color('whiteonblue'))
        for instance in check_instances:
            instance.update()
            instance.reset_ssh_connection()
            try:
                bad_vols = instance.get_unsynced_volumes()
                if bad_vols:
                    errmsg = str(instance.id) + "Unsynced vols found:"
                    for bad_vol in bad_vols:
                        errmsg = errmsg + str(bad_vol.id) + ":" + str(bad_vol.status) + ","
                    errmsg = errmsg + "\n"
            except Exception, ve:
                errmsg = errmsg + "\n" + str(ve)
            if errmsg:
                raise Exception(errmsg)
            for vol in instance.attached_vols:
                try:
                    md5before = vol.md5
                    md5now = instance.md5_attached_euvolume(vol,updatevol=False)
                    if md5before != md5now:
                        errmsg += str(instance.id) +"Volume:" + str(vol.id) \
                                    + "has different md5 sum after service interruption. Before:'" \
                                    + str(md5before) + "' - vs - '" + str(md5now) + "'"
                    vol.length = write_length
                    instance.vol_write_random_data_get_md5(vol,length=write_length, overwrite=True)
                    instance.sys('sync',code=0)
                except Exception, e:
                    errmsg += str(instance.id) + "Volume:" + \
                              str(vol.id) + ", error while using vol post service interruption, err: "+str(e)
        if errmsg:
            raise Exception(errmsg)
        self.status("Read/write test passed, now testing detachment of volumes...",
                    testcolor=TestColor.get_canned_color('whiteonblue'))
        for instance in self.instances:
            for vol in instance.attached_vols:
                try:
                    instance.detach_euvolume(vol)
                except Exception, e:
                    errmsg += str(instance.id) + "Volume:" + str(vol.id) + "Error while detaching, err:" + str(e)
        if errmsg:
            raise Exception(errmsg)
        self.status("Attached volume checks post service interruption have passed.",
                    testcolor=TestColor.get_canned_color('whiteonblue'))



    def test2_post_service_interuption_check_volume_from_snapshots(self):

        """
        Definition: Attempts to check volume creation from snapshots created before SC restart.
        """
        self.status("Checking creation of volumes from pre-existing snapshots post service interruption...",
                    testcolor=TestColor.get_canned_color('whiteonblue'))
        vols = []
        if not self.snapshots:
            raise Exception("self.snapshots not populated?")
        for snap in self.snapshots:
            for zone in self.zones:
                vols.extend(self.tester.create_volumes(zone, snapshot=snap, monitor_to_state=None))
        #Now monitor created volumes to state available, use a large timeout per gig here.
        self.volumes.extend(self.tester.monitor_created_euvolumes_to_state(vols, state='available', timepergig=360))
        self.status("Done creating volumes from pre-existing snapshots post service interruption.",
                    testcolor=TestColor.get_canned_color('whiteonblue'))


    def test3_post_service_interuption_check_volume_attachment_of_new_vols_from_old_snaps(self):
        vols = []
        errmsg = ""
        for snap in self.snapshots:
            vols.extend(self.tester.get_volumes(status='available', snapid=snap.id))

        if not vols:
            raise Exception("No vols were found as created from previous snapshots")
        if not self.instances:
            raise Exception('No instances to use for this test')
        if not self.snapshots:
            raise Exception('No snapshots to use for this test')
        for vol in vols:
            vol.update()
        availvols = copy.copy(vols)
        self.status('Attempting to attach the following volumes:',testcolor=TestColor.get_canned_color('whiteonblue'))
        self.tester.print_euvolume_list(availvols)
        #Iterate through the volumes, and attach them all to at least one instance in each zone.
        for instance in self.instances:
            if not availvols:
                break
            for vol in availvols:
                if vol.zone == instance.placement:
                    availvols.remove(vol)
                    try:
                        try:
                            instance.attach_volume(vol, timeout=90)
                        except ec2ops.VolumeStateException, vse:
                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                        testcolor=TestColor.get_canned_color('failred'))
                            time.sleep(10)
                            self.tester.monitor_euvolumes_to_status([vol],status='in-use',attached_status='attached',timeout=60)
                        for snap in self.snapshots:
                            snap.update()
                            if vol.snapshot_id == snap.id:
                                if vol.md5len != snap.eutest_volume_md5len:
                                    self.debug('Need to adjust md5sum for length of snapshot...')
                                    vol.md5len = snap.eutest_volume_md5len
                                    instance.vol_write_random_data_get_md5(vol,length=snap.eutest_volume_md5len)
                                if vol.md5 != snap.eutest_volume_md5:
                                    errmsg += "Volume:" + str(vol.id) + " MD5 did not match snapshots " \
                                              + str(snap.id) + ": snapmd5:" + str(snap.eutest_volume_md5) \
                                              + " --vs volmd5:-- " + str(vol.md5)
                    except Exception, e:
                        errmsg += str(instance.id) +"Volume:" + str(vol.id) \
                              + " error when attaching and comparing md5, err:" + str(e)
        self.status('Volume status post attachment operation:',testcolor=TestColor.get_canned_color('whiteonblue'))
        self.tester.print_euvolume_list(vols)
        if errmsg:
            raise Exception(errmsg)




    def test4_post_service_interuption_check_snapshot_creation(self):
        self.status('Attempting to verify snapshot creation post reboot',\
                    testcolor=TestColor.get_canned_color('whiteonblue'))
        testvols = []
        testsnaps = []
        for zone in self.zones:
            for vol in self.volumes:
                if vol.zone == zone:
                    testvols.append(vol)
                    break
        for vol in testvols:
            testsnaps.extend(self.tester.create_snapshots(vol, monitor_to_completed=False))
        wait_on_progress = int(len(testsnaps)*25)
        self.tester.monitor_eusnaps_to_completed(testsnaps,wait_on_progress=wait_on_progress)
        self.snapshots.extend(testsnaps)

    def test5_post_service_interruption_check_bfebs_instances(self):
        """
        Definition: Attempts to check the state of bfebs instances launched in this test prior
        to restarting the SC.
        """
        for instance in self.bfebs_instances:
            last_state = instance.state
            instance.update()
            if instance.state != last_state:
                raise Exception('Instance state before SC restart: '
                                + str(last_state) + " != post restart:"
                                + str(instance.state))
            self.test1_post_service_interruption_check_attached_volumes(check_vols=self.bfebs_volumes,
                                                                        check_instances=self.bfebs_instances)


if __name__ == "__main__":
    testcase = Ebs_Persistance_Tests()

    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list

    instance_store_only_tests = [ 'pretest1_pre_service_restart_launch_test_instances',
                                    'pretest2_pre_service_restart_create_volume_resources',
                                    'pretest3_pre_service_restart_attach_all_volumes',
                                    'pretest4_pre_service_restart_create_snap_per_zone',
                                    'reboot_sc_machine_verify_post_reboot',
                                    'test1_post_service_interruption_check_attached_volumes',
                                    'test2_post_service_interuption_check_volume_from_snapshots',
                                    'test3_post_service_interuption_check_volume_attachment_of_new_vols_from_old_snaps',
                                    'test4_post_service_interuption_check_snapshot_creation']
    bfebs_test_list = [ 'pretest1_pre_service_restart_launch_test_instances',
                        'pretest2_pre_service_restart_create_volume_resources',
                        'pretest3_pre_service_restart_attach_all_volumes',
                        'pretest4_pre_service_restart_create_snap_per_zone',
                        'pretest5_pre_service_restart_launch_bfebs_test_instances',
                        'pretest6_pre_service_restart_create_bfebs_volume_resources',
                        'pretest7_per_service_restart_attach_vols_to_bfebs_instances',
                        'reboot_sc_machine_verify_post_reboot',
                        'test1_post_service_interruption_check_attached_volumes',
                        'test2_post_service_interuption_check_volume_from_snapshots',
                        'test3_post_service_interuption_check_volume_attachment_of_new_vols_from_old_snaps',
                        'test4_post_service_interuption_check_snapshot_creation',
                        'test5_post_service_interruption_check_bfebs_instances']

    list = testcase.args.tests
    if not list:
        if testcase.args.testbfebs:
            list = bfebs_test_list
        else:
            list = instance_store_only_tests


    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,eof=True,clean_on_exit=testcase.clean_on_exit)
    exit(result)



#!/usr/bin/python

import time
import uuid
import random

class TimeUnit:
    def __init__(self):
        self._start = time.time()
        self._elapsed_time = 0

    def end(self):
        self._elapsed_time = time.time() - self._start

    def elapsed(self):
        return self._elapsed_time

class Timer:
    def __init__(self, logfile="/tmp/eutester", debug=False):
        self._timers = {}
        self._log = open(logfile, "a+b")
        self._debug = debug
        if self._debug:
            print("logging elapsed time to: " + logfile)

    def start(self):
        index = "%s" % (uuid.uuid4())
        self._timers[index] = TimeUnit()
        return index

    def end(self, id, msg):
        self._timers[id].end()
        self._log.write(msg + "\tid: %s\t%fms\n" % (id, self._timers[id].elapsed()*1000))
        if self._debug:
            print("elapsed: " + self._timers[id].elapsed()*1000 + "ms")


    def finish(self):
        self._log.close()




#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: Vic Iglesias vic.iglesias@eucalyptus.com
# Author: Shaon shaon@eucalyptus.com


import os
import time
from eucaops import Eucaops
from eutester.euinstance import EuInstance
from eutester.eutestcase import EutesterTestCase
import random
from eutester.euvolume import EuVolume
from testcases.cloud_user.instances.bfebstest import BFEBSBasics


class MigrationTest(EutesterTestCase):
    def __init__(self, extra_args= None):
        self.setuptestcase()
        self.setup_parser()
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.parser.add_argument('--imgurl',
                        help="BFEBS Image to splat down", default=None)
        self.get_args()
        self.tester = Eucaops( config_file=self.args.config, password=self.args.password)
        self.numberOfNodes = self.tester.service_manager.get_all_node_controllers()
        if len(self.numberOfNodes) < 2:
            self.tester.debug("Not enough NCs to test instance migration.")
            exit(0)

        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )

        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)

        self.image = self.args.emi
        if not self.image:
            self.image = self.tester.get_emi(root_device_type="instance-store")
        self.numberOfResources = 3
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name

        try:
            self.tester.get_emi(root_device_type="ebs")
        except:
            bfebs = self.do_with_args(BFEBSBasics)
            bfebs.RegisterImage()

    def clean_method(self):
        self.tester.cleanup_artifacts()

    def MigrationBasic(self, volume=None):
        enabled_clc = self.tester.service_manager.get_enabled_clc().machine
        self.reservation = self.tester.run_instance(self.image, username=self.args.instance_user, keypair=self.keypair.name, group=self.group.name, zone=self.zone)
        instance = self.reservation.instances[0]
        assert isinstance(instance, EuInstance)
        volume_device = None
        if volume is not None:
            volume_device = instance.attach_euvolume(volume)

        self.tester.service_manager.populate_nodes()
        source_nc = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id)[0]
        enabled_clc.sys( "source " + self.tester.credpath + "/eucarc &&" +
                         self.tester.eucapath + "/usr/sbin/euca-migrate-instances -i " + instance.id )

        def wait_for_new_nc():
            self.tester.service_manager.populate_nodes()
            destination_nc = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id)[0]
            return source_nc.hostname == destination_nc.hostname

        self.tester.wait_for_result(wait_for_new_nc, False, timeout=600, poll_wait=60)
        self.assertTrue(self.tester.ping(instance.public_dns_name), 'Could not ping instance')

        if volume_device:
            instance.sys("ls " + volume_device, code=0)

        destination_nc = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id)[0]
        if destination_nc.machine.distro.name is not "vmware":
            destination_nc.machine.sys("virsh list | grep " + instance.id, code=0)
        else:
            destination_nc.machine.sys("esxcli vm process list | grep " + instance.id, code=0)

        self.tester.terminate_instances(reservation=self.reservation)
        if volume is not None:
            self.tester.delete_volume(volume)

    def MigrationInstanceStoreWithVol(self):
        volume = self.tester.create_volume(zone=self.zone)
        assert isinstance(volume, EuVolume)
        self.MigrationBasic(volume)

    def MigrationBasicEBSBacked(self, volume=None):
        self.image = self.tester.get_emi(root_device_type="ebs")
        self.MigrationBasic(volume)

    def MigrationBasicEBSBackedWithVol(self):
        volume = self.tester.create_volume(zone=self.zone)
        assert isinstance(volume, EuVolume)
        self.MigrationBasicEBSBacked(volume)

    def MigrateToDest(self):
        enabled_clc = self.tester.service_manager.get_enabled_clc().machine
        self.reservation = self.tester.run_instance(self.image, username=self.args.instance_user, keypair=self.keypair.name, group=self.group.name, zone=self.zone)
        instance = self.reservation.instances[0]
        self.tester.service_manager.populate_nodes()
        self.source_nc = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id)[0]

        all_nc = self.tester.service_manager.get_all_node_controllers()
        self.destination_nc = None

        for nc in all_nc:
            if nc.machine.hostname != self.source_nc.machine.hostname:
                self.destination_nc = nc
                enabled_clc.sys("source " + self.tester.credpath + "/eucarc && " +
                                    self.tester.eucapath + "/usr/sbin/euca-migrate-instances -i " +
                                    instance.id + " --dest " + self.destination_nc.machine.hostname)

                def wait_for_new_nc():
                    self.tester.service_manager.populate_nodes()
                    self.instance_node = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id)[0]
                    return self.instance_node.hostname == self.destination_nc.hostname

                self.tester.wait_for_result(wait_for_new_nc, True, timeout=600, poll_wait=60)
                self.assertTrue( self.tester.ping(instance.public_dns_name), 'Could not ping instance')

        # migrate the instance to it's original source node
        self.destination_nc = self.source_nc
        enabled_clc.sys("source " + self.tester.credpath + "/eucarc && " +
                            self.tester.eucapath + "/usr/sbin/euca-migrate-instances -i " +
                            instance.id + " --dest " + self.destination_nc.machine.hostname)

        self.tester.wait_for_result(wait_for_new_nc, True, timeout=600, poll_wait=60)
        self.assertTrue(self.tester.ping(instance.public_dns_name), 'Could not ping instance')

        self.tester.terminate_instances(reservation=self.reservation)

    def MigrationToDestEBSBacked(self):
        self.image = self.tester.get_emi(root_device_type="ebs")
        self.MigrateToDest()

    def EvacuateNC(self, volume_list = []):
        instance_list = []
        enabled_clc = self.tester.service_manager.get_enabled_clc().machine
        self.nodes = self.tester.service_manager.populate_nodes()
        # pop out one NC to fill in
        self.source_nc = self.nodes.pop()

        def set_state(node, state):
            # retrying, see EUCA-6389
            while node.state != state:
                self.tester.debug(node.hostname + ": SET STATE TO " + state)
                enabled_clc.sys("euca-modify-service -s " + state + " " + node.hostname)
                self.tester.sleep(10)
                tmpnodes = self.tester.service_manager.populate_nodes()
                for tmpnode in tmpnodes:
                    if tmpnode.hostname == node.hostname:
                        node = tmpnode

        # stop all the NCs
        for node in self.nodes:
            set_state(node, "STOPPED")

        self.image = self.tester.get_emi(root_device_type="instance-store")
        self.reservation = self.tester.run_instance(self.image, min=3, max=3, username=self.args.instance_user, keypair=self.keypair.name, group=self.group.name, zone=self.zone)

        for i in xrange(3):
            instance = self.reservation.instances[i]
            instance_list.append(instance)
            assert isinstance(instance, EuInstance)
            volume_device = None
            if volume_list:
                volume_device = instance.attach_euvolume(volume_list[i])

        self.nodes = self.tester.service_manager.populate_nodes()
        # start all the NCs
        for node in self.nodes:
            if node.hostname is not self.source_nc.hostname:
                set_state(node, "ENABLED")

        self.nodes = self.tester.service_manager.populate_nodes()
        # evacuate source NC
        enabled_clc.sys("source " + self.tester.credpath + "/eucarc && " +
                            self.tester.eucapath + "/usr/sbin/euca-migrate-instances --source " +
                        self.source_nc.machine.hostname)

        def wait_for_evacuation():
            self.tester.service_manager.populate_nodes()
            if self.source_nc.machine.distro.name is "vmware":
                emptyNC = self.source_nc.sys("esxcli vm process list | grep 'Display Name' | awk '{print $3}'")
            else:
                emptyNC = self.source_nc.get_virsh_list()
            return len(emptyNC) == 0

        self.tester.wait_for_result(wait_for_evacuation, True, timeout=600, poll_wait=60)

        for inst in instance_list:
            self.assertTrue(self.tester.ping(inst.public_dns_name), 'Could not ping instance')

        self.tester.terminate_instances(reservation=self.reservation)
        if volume_list:
            self.tester.delete_volumes(volume_list)

    def EvacuateNCWithVol(self):
        volume_list = []
        for i in xrange(self.numberOfResources):
            volume = self.tester.create_volume(zone=self.zone)
            assert isinstance(volume, EuVolume)
            volume_list.append(volume)
        self.EvacuateNC(volume_list)

    def EvacuateNCAllEBS(self):
        self.image = self.tester.get_emi(root_device_type="ebs")
        self.EvacuateNC()

if __name__ == "__main__":
    testcase = MigrationTest()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["MigrationBasic", "MigrationInstanceStoreWithVol", "MigrationBasicEBSBacked", "MigrationBasicEBSBackedWithVol", "MigrateToDest", "MigrationToDestEBSBacked", "EvacuateNC", "EvacuateNCWithVol", "EvacuateNCAllEBS"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append(testcase.create_testunit_by_name(test))

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


import threading
import time
import re
import sys
import Queue 
from eutester.sshconnection import SshConnection
from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import EutesterTestResult
import types


my_queue = Queue.Queue()


class Path_Controller(EutesterTestCase):
    #a unique comment to add to iptables rules to signify the rule was added by this test
    ipt_msg = "eutester block data to san"
    
    def __init__(self,
                 node=None,
                 queue=None,
                 interval=None,
                 restore_time=None,
                 sp_ip_list=None,
                 path_iterations=None):
        self.queue = queue or my_queue
        self.setuptestcase()
        if not (node or queue or sp_ip_list):
            self.setup_parser(testname='Path_Controller', vmtype=False,zone=False, keypair=False,emi=False,credpath=False,
                              description='Run multipath failover script')
            self.parser.add_argument('--clear_rules', help='If set will clear all eutester applied rules matching ipt_msg string',action='store_true', default=False)
            self.parser.add_argument('--hostname', help='String representing host address or FQDN',default=None)
            self.parser.add_argument('--clear_on_exit', help='If set will clear rules on exit',action='store_true', default=False)
            self.parser.add_argument('--username', help='String representing username for host login, default:root',default='root')
            self.parser.add_argument('--keypath', help='String representing local path to host ssh key',default=None)
            self.parser.add_argument('--interval', help='Integer representing seconds between path failover',default=30)
            self.parser.add_argument('--restore_time', help='Integer representing seconds to allow path recovery',default=15)
            self.parser.add_argument('--sp_ip_list', help='String with SP addrs, comma delimited',default=None)
            self.parser.add_argument('--path_iterations', help='Number of times to iterate through sp_ip_list when \
                                     blocking paths. "0" loops forever', default=2)
            self.get_args()

        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        """
        for kw in kwargs:
            print 'Setting kwarg:' + str(kw) + " to " + str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        """

        if self.has_arg('path_iterations'):
            self.path_iterations =  self.args.path_iterations
        else:
            self.path_iterations =  path_iterations or 2
        self.total_path_iterations = 0
        self.node = node
        if self.node:
            self.host = node.hostname
            self.ssh = self.node.machine.ssh
        else:
            self.host = self.args.hostname
            self.ssh = SshConnection(self.args.host,
                                     keypath=self.args.keypath,
                                     password=self.args.password,
                                     username=self.args.username,
                                     debugmethod=self.debug,
                                     verbose=True)
        self.sys = self.ssh.sys
        self.cmd = self.ssh.cmd

        if self.has_arg('interval'):
            self.interval = int(self.args.interval)
        else:
            self.interval = interval or 30

        if self.has_arg('restore_time'):
            self.restore_time = int(self.args.restore_time)
        else:
            self.restore_time = restore_time or 15
        self.start = time.time()
        self.lastblocked = None
        self.blocked = []
        self.get_sp_ip_list(sp_ip_list=sp_ip_list)
        self.timer = None
        self.last_clear_attempt_time = 0
        self.last_cleared_time = 0
        self.last_block_time = 0
        self.debug('Path_Controller init:' \
                    + "\nhost:" + str(self.host) \
                    + "\nsp_ipt_list:" + str(self.sp_ip_list) \
                    + "\ninterval:" + str(self.interval) \
                    + "\nrestore_time:" + str(self.restore_time))


    def get_tag(self):
        return str(self.ipt_msg) + ", time:" + str(time.time())

    def get_sp_ip_list(self,sp_ip_list=None, sp_ip_list_string=None):
        ret_list = []
        if sp_ip_list and isinstance(sp_ip_list, types.ListType):
            ret_list = sp_ip_list
        else:
            sp_ip_list_string = sp_ip_list_string or self.args.sp_ip_list
            if not sp_ip_list_string:
                raise Exception('No sp_ip_list provided')
            for path in str(sp_ip_list_string).split(','):
                for part in path.split(':'):
                    #Don't use iface strings for this test...
                    if not re.search('iface', part):
                        ret_list.append(part)
        if not ret_list:
            raise Exception('No sp_ip_list parsed from sp_ip_list_string:'+str(sp_ip_list_string))
        self.sp_ip_list = ret_list
        return ret_list

    def get_eutester_current_block_rules(self):
        output = self.sys('iptables -L -n --line-numbers | grep "'+str(self.ipt_msg)+'"')
        return output

    def clear_all_eutester_rules(self, retry=True,timeout=60):
        #self.debug('Clear_all_eutester_rules...')
        self.last_clear_attempt_time = time.time()
        #clears all rules which match the description string ipt_msg
        start = time.time()
        elapsed = 0
        output = self.sys('iptables -L -n --line-numbers | grep "'+str(self.ipt_msg)+'"')
        while output and elapsed < timeout:
            elapsed = int(time.time()-start)
            self.debug("Attempting to remove "+str(len(output))+" eutester rules from system. Elapsed: "+str(elapsed)+'/'+str(timeout))
            for line in output:
                self.debug('Clearing rule: '+str(line))
                rule_number = str(line).split()[0]
                self.sys('iptables -D OUTPUT '+str(rule_number))
                self.sys('iptables -D INPUT '+str(rule_number))
            output = self.sys('iptables -L -n --line-numbers | grep "'+str(self.ipt_msg)+'"')
            if not output:
                self.last_cleared_time = time.time()
            else:
                self.last_cleared_time = 0
                if not retry:
                    return len(output)
                time.sleep(2)
        if not output:
            self.last_cleared_time = time.time()
        return len(output)

        
    def set_timer(self, interval=30, cb=None, *args):
        self.debug("set_timer starting with interval "+str(interval)+"...")
        if self.timer:
            self.timer.cancel()
            self.timer = None
        self.timer = threading.Timer(interval, cb, args)
    
    def is_path_blocked(self, addr):
        self.debug('Checking iptables for path state: '+str(addr)+' ...')
        if re.match('\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', addr):
            out = self.sys('iptables -L -n ',code=0)
        else:
            out = self.sys('iptables -L ',code=0)
        for line in out:
            if re.search(self.ipt_msg, line) and re.match("^DROP", line):
                #make sure we don't get a partial match...
                lines = line.rstrip().split()
                for word in lines:
                    if re.match(addr, word):
                        return True
        return False


    def get_blocked_paths(self):
        list = []
        for ip in self.sp_ip_list:
            if self.is_path_blocked(ip):
                list.append(ip)
        self.blocked = list
        return list


    def restore_path(self, addr):
        self.debug("restore_path starting for path:'"+str(addr)+"'")
        if self.is_path_blocked(addr):
            self.debug('Attempting to remove iptables rule blocking traffic to: "'+str(addr)+'"')
            self.sys('iptables -D OUTPUT -j DROP -d '+str(addr), code=0)
            self.wait(2)
            #time.sleep(2)
            if not self.can_ping_path(addr):
                raise Exception("Could not ping path:"+str(addr)+" after restoring path rule")
        if addr in self.blocked:
            self.blocked.remove(addr)
    
    def restore_paths(self, addr_list ):
        self.debug('restore_paths starting, for list:'+str(",").join(addr_list))
        for addr in addr_list:
            self.restore_path(addr)  
             
    def block_path(self,addr):
        self.debug('Attempting to add iptables rule blocking traffic to: "'+str(addr)+'"')
        #First delete rule before adding in the case a duplicate exists
        try:
            self.sys('iptables -D OUTPUT -j DROP -d '+str(addr))
        except: pass
        try:
            self.sys('iptables -D INPUT -j DROP -d '+str(addr))
        except: pass
        #Add rule to block...
        self.sys('iptables -A OUTPUT -j DROP -d '+str(addr)+' -m comment --comment "'+str(self.ipt_msg)+'"', code=0)
        self.sys('iptables -A INPUT -j DROP -d '+str(addr)+' -m comment --comment "'+str(self.ipt_msg)+'"', code=0)
        if not addr in self.blocked:
            self.blocked.append(addr)
        self.last_block_time = time.time()
        self.lastblocked = addr

    def can_ping_path(self, addr, count=1, timeout=10):
        self.debug('can_ping_path starting:'+str(addr)+", count:"+str(count)+", timeout:"+str(timeout))
        try:
            self.sys('ping -c '+str(count)+' -t '+str(timeout), code=0)
            return True
        except:pass
        return False

    def block_next_path(self, lastblocked=None):
        last_blocked = lastblocked or self.lastblocked
        if last_blocked:
            index = self.sp_ip_list.index(last_blocked)
            #If were at the end of the list return to index 0
            if index == (len(self.sp_ip_list)-1):
                block = self.sp_ip_list[0]
                self.total_path_iterations += 1
            else:
                block = self.sp_ip_list[index+1]
        else:
            block = self.sp_ip_list[0]
        self.debug("block_next_path, attempting to block path:"+str(block)+", set timer to:"+str(self.interval))
        self.block_path(block)
        return block

    def get_blocked_string(self):
        out = ""
        for addr in self.get_blocked_paths():
            out += str(addr) + ","
        return out

    def reset(self):
        if self.timer:
            self.timer.cancel()
        self.clear_all_eutester_rules()


    def wait(self,seconds):
        seconds = int(seconds)
        self.debug("Waiting for '"+str(seconds)+"' seconds...", traceback=2)
        sys.stdout.write('Elapsed:')
        for x in xrange(0,seconds):
            if (x%seconds == 0):
                sys.stdout.write(str(x))
            else:
                sys.stdout.write('-')
            sys.stdout.flush()
            time.sleep(1)
        print(str(seconds))




    '''
    def cycle_paths(self, lastblocked=None, wait_for_clear=True, timeout_on_clear=30, set_timer=0):
        self.debug('block_next_path starting...')
        try:
            #iterate through sp ip list, block the next available ip in the list. 
            #if we've reached the end of the list restore all paths for an interval period

            #If we only have a single path, or no paths return...
            if not self.sp_ip_list or len(self.sp_ip_list) == 1:
                return

            blocked_paths = self.get_blocked_paths()
            if blocked_paths:
                self.status('Clearing all blocked paths prior to cycling paths')
                self.clear_all_eutester_rules()

            #If there was a blocked path(s) then wait for restore period...
            if wait_for_clear:
                elapsed = 0
                start = time.time()
                while blocked_paths and elapsed < timeout_on_clear:
                    self.clear_all_eutester_rules()
                    blocked_paths = self.get_blocked_paths()
                    if blocked_paths:
                        time.sleep(1)
                    elapsed = int(time.time() - start)
                if blocked_paths:
                    raise Exception('Could not clear blocked paths within ' + str(elapsed) + ' seconds; ' + ",".join(blocked_paths))
            block = self.block_next_path(lastblocked=lastblocked)
            args = [block,wait_for_clear]
            if set_timer:
                self.set_timer(set_timer, self.block_single_path_cycle, args)
                self.timer.start()
        except KeyboardInterrupt, k:
            if self.timer:
                self.timer.cancel()
                raise Exception('Caught keyboard interrupt...')
        nqstr = 'Blocking:' +str(self.blocked)
        self.queue.put(nqstr)
        '''


 
        

       
            
            
        
            




#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
__author__ = 'clarkmatthew'


from winrm.protocol import Protocol
from winrm.exceptions import WinRMTransportError
from isodate.isoduration import duration_isoformat
from datetime import timedelta
import StringIO
import traceback
import socket
import copy
import sys
import time
import re


class Winrm_Connection:

    def __init__(self,
                 hostname,
                 username,
                 password,
                 port=5985,
                 protocol='http',
                 transport='plaintext',
                 default_command_timeout=600,
                 url=None,
                 debug_method=None,
                 verbose=True):
        self.debug_method = debug_method
        self.hostname = hostname
        self.username = username
        self.password = password
        self.port = int(port)
        self.protocol = protocol
        self.transport = transport
        self.default_command_timeout = default_command_timeout #self.convert_iso8601_timeout(default_command_timeout)
        self.url = url or str(protocol)+"://"+str(hostname)+":"+str(port)+"/wsman"
        self.winproto = self.get_proto()
        self.shell_id = None
        self.command_id = None
        self.last_used = None

        self.verbose = verbose


    def get_proto(self):
        self.debug('Creating winrm connection:' + str(self.hostname) + ":" + str(self.port) + ", Username:" + str(self.username) + ', Password:' + str(self.password))
        winproto = Protocol(endpoint=self.url,transport=self.transport,username=self.username,password=self.password)
        #winproto.transport.timeout = self.default_command_timeout
        return winproto

    def convert_iso8601_timeout(self, timeout):
        #convert timeout to ISO8601 format
        return duration_isoformat(timedelta(int(timeout)))

    def debug(self, msg):
        if self.debug_method:
            self.debug_method(msg)
        else:
            print(msg)

    def reset_shell(self, timeout=None, retries=5):
        retry = 0
        tb = ""
        e = None
        self.close_shell()
        timeout = timeout or self.default_command_timeout
        self.winproto.transport.timeout = timeout #self.default_command_timeout
        #self.debug('reset_shell connection, Host:' + str(self.hostname) + ":" + str(self.port) + ", Username:" + str(self.username) + ', Password:' + str(self.password))
        while retry < retries:
            retry += 1
            try:
                self.shell_id = self.winproto.open_shell()
                return self.shell_id
            except WinRMTransportError, wte:
                print "Failed to open shell on attempt#:" + str(retry) + "/" + str(retries)+ ", err:" + str(wte)
                if retry < retries:
                    time.sleep(5)
            except Exception, e:
                tb = self.get_traceback()
                errmsg = "Error caught while reseting winrm shell:" +str(e)
                self.debug("Error caught while reseting winrm shell:" +str(e))
        self.debug(str(tb))
        raise Exception('Could not open shell to ' + str(self.url) + str(e))

    def cmd(self, command, console_mode_stdin=True, skip_cmd_shell=False, timeout=None, verbose=None):
        errmsg = ""
        if verbose is None:
            verbose = self.verbose
        orig_cmd = copy.copy(command)
        arguments = command.split(' ')
        command = arguments.pop(0)
        self.command_id = None

        #if timeout is not None:
            #convert timeout to ISO8601 format
            #timeout = self.convert_iso8601_timeout(timeout)
        self.reset_shell(timeout=timeout)
        try:
            self.command_id= self.winproto.run_command(self.shell_id,
                                      command,
                                      arguments=arguments,
                                      console_mode_stdin=console_mode_stdin,
                                      skip_cmd_shell=skip_cmd_shell)
            self.debug('winrm timeout:' + str(timeout) + ', cmd:' + str(orig_cmd))
            if timeout is not None:
                sockdefault = socket.getdefaulttimeout()
                socket.setdefaulttimeout(timeout)
                stdout, stderr, statuscode = self.get_timed_command_output(self.shell_id, self.command_id, active_timeout=timeout)
            else:
                stdout, stderr, statuscode = self.winproto.get_command_output(self.shell_id, self.command_id)
            self.debug( 'Command:"' + str(orig_cmd) + '" , Done.')
        except WinRMTransportError as wte:
            errmsg = str(wte)
        except CommandTimeoutException as cte:
            self.debug(str(cte))
            errmsg = 'timed out'
        finally:
            try:
                #self.winproto.transport.timeout = self.default_command_timeout
                if timeout is not None:
                    socket.setdefaulttimeout(sockdefault)
                self.winproto.cleanup_command(self.shell_id, self.command_id)
            except: pass
            self.close_shell()
        if errmsg:
            if re.search('timed out', errmsg, re.IGNORECASE):
                raise CommandTimeoutException('ERROR: Timed out after:' +
                                              str(self.winproto.transport.timeout) +
                                              ', Cmd:"' + str(orig_cmd))
            else:
                raise Exception(errmsg)
        if verbose:
            self.debug("\n" + str(stdout) + "\n" + str(stderr))
        return {'stdout':stdout, 'stderr':stderr, 'statuscode':statuscode}



    def get_timed_command_output(self, shell_id, command_id, active_timeout=0):
        """
        Get the Output of the given shell and command
        @param string shell_id: The shell id on the remote machine.  See #open_shell
        @param string command_id: The command id on the remote machine.  See #run_command
        @param int active_timeout: Time out used during an active session. For example as the shell is actively returning
                                  data, but we want to timeout anyways. See cmd timeout for idle timeout where no
                                  data has been read.
        """
        stdout_buffer, stderr_buffer = [], []
        command_done = False
        start = time.time()
        while not command_done:
            elapsed = time.time()-start
            if active_timeout and (elapsed > active_timeout):
                raise CommandTimeoutException('Active timeout fired after:' + str(elapsed))
            stdout, stderr, return_code, command_done = \
                self.winproto._raw_get_command_output(shell_id, command_id)
            stdout_buffer.append(stdout)
            stderr_buffer.append(stderr)
        return ''.join(stdout_buffer), ''.join(stderr_buffer), return_code


    def close_shell(self):
        if self.shell_id:
            self.winproto.close_shell(self.shell_id)
        self.shell_id = None

    def sys(self, command, include_stderr=False, listformat=True, carriage_return=False, timeout=None, code=None, verbose=None):
        ret = []
        if verbose is None:
            verbose = self.verbose
        output = self.cmd(command, timeout=timeout, verbose=verbose )
        if code is not None and output['statuscode'] != code:
            raise CommandExitCodeException('Cmd:' + str(command) + ' failed with status code:'
                                               + str(output['statuscode'])
                                               + "\n, stdout:" + str(output['stdout'])
                                               + "\n, stderr:" + str(output['stderr']))
        ret = output['stdout']
        if ret:
            if not carriage_return:
                #remove the '\r' chars from the return buffer, leave '\n'
                ret = ret.replace('\r','')
            if listformat:
                ret = ret.splitlines()
            if include_stderr:
                ret = ret.extend(output['stderr'].splitlines())
        return ret

    @classmethod
    def get_traceback(cls):
        '''
        Returns a string buffer with traceback, to be used for debug/info purposes.
        '''
        try:
            out = StringIO.StringIO()
            traceback.print_exception(*sys.exc_info(),file=out)
            out.seek(0)
            buf = out.read()
        except Exception, e:
                buf = "Could not get traceback"+str(e)
        return str(buf)



class CommandExitCodeException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class CommandTimeoutException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


#!/usr/bin/env python
#
#
# Description:  This script encompasses test cases/modules concerning instance specific behavior and
#               features for Eucalyptus.  The test cases/modules that are executed can be
#               found in the script under the "tests" list.


import time
from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
import os
import random

class InstanceRestore(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config_file, password=self.args.password)
        self.tester.poll_count = 120

        ### Add and authorize a group for the instance
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        self.image = self.tester.get_emi()
        self.reservation = None
        self.private_addressing = False
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name
        clcs = self.tester.get_component_machines("clc")
        if len(clcs) is 0:
            raise Exception("No CLC found")
        else:
            self.clc = clcs[0]
        self.cur_time = str(int(time.time()))
        self.ncs = self.tester.get_component_machines("nc")

    def clean_method(self):
        ncs = self.tester.get_component_machines("nc")
        for nc in ncs:
            nc.sys("service eucalyptus-nc start")

        ### RESET vmstate properties
        self.tester.modify_property("cloud.vmstate.instance_timeout","60")
        self.tester.modify_property("cloud.vmstate.terminated_time","60")
        for nc in self.ncs:
            nc.sys("service eucalyptus-nc start")
        self.tester.cleanup_artifacts()
        try:
            image = self.tester.get_emi(self.image)
        except Exception,e:
            self.tester.register_image(image_location=self.image.location,
                                       ramdisk=self.image.ramdisk_id,
                                       kernel=self.image.kernel_id)

    def RestoreLogic(self):
        self.tester.modify_property("cloud.vmstate.instance_timeout","1")
        self.tester.modify_property("cloud.vmstate.terminated_time","1")

        self.reservation = self.tester.run_instance(self.image, keypair=self.keypair.name, group=self.group.name, zone=self.zone)

        for nc in self.ncs:
            nc.sys("service eucalyptus-nc stop")

        ### Wait for instance to show up as terminating
        self.tester.wait_for_reservation(self.reservation, state="terminated", timeout=600)

        instance_under_test = None
        for instance in self.reservation.instances:
            instance_under_test = instance
            instance.terminate()

        self.tester.deregister_image(self.image)

        for nc in self.ncs:
            nc.sys("service eucalyptus-nc start")

        found = False
        while not found:
            try:
                if not self.tester.ec2.get_all_instances(instance_ids=[instance_under_test.id]):
                    raise Exception("Unable to find instance")
                else:
                    found = True
            except Exception, e:
                self.tester.debug(str(instance_under_test) + " not found yet")
            self.tester.sleep(10)

        self.tester.wait_for_instance(instance_under_test, state="running", timeout=600)

        for instance in self.reservation.instances:
            instance.sys("uname -r", code=0)



if __name__ == "__main__":
    testcase = InstanceRestore()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or ["RestoreLogic"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



#!/usr/bin/python

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

class SampleTest(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config,password=self.args.password)

    def clean_method(self):
        pass

    def MyTest(self):
        """
        This is where the test description goes
        """
        for machine in self.tester.get_component_machines("clc"):
            machine.sys("ifconfig")

if __name__ == "__main__":
    testcase = SampleTest()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["MyTest"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#!/usr/bin/env python
import os
import time

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eutester.machine import Machine


class SOSreport(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.start_time = int(time.time())
        self.parser.add_argument("--ticket-number", default=str(self.start_time))
        self.parser.add_argument("--timeout", default=1200, type=int)
        self.parser.add_argument("--remote-dir", default="/root/")
        self.parser.add_argument("--local-dir", default=os.getcwd())
        self.parser.add_argument("--package-url", default="http://downloads.eucalyptus.com/software/tools/centos/6/x86_64/eucalyptus-sos-plugins-0.1.5-0.el6.noarch.rpm")
        self.get_args()
        self.remote_dir = self.args.remote_dir + "/euca-sosreport-" + self.args.ticket_number + "/"
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config,password=self.args.password, download_creds=False)

    def clean_method(self):
        pass

    def Install(self):
        """
        This is where the test description goes
        """
        for machine in self.tester.get_component_machines():
            assert isinstance(machine, Machine)
            if machine.distro.name is "vmware":
                continue
            machine.install("sos")
            machine.sys("yum install -y " + self.args.package_url)

    def Run(self):
        error_msg = ""
        for machine in self.tester.get_component_machines():
            try:
                assert isinstance(machine, Machine)
                if machine.distro.name is "vmware":
                    continue
                machine.sys("mkdir -p " + self.args.remote_dir)
                machine.sys("sosreport --batch --skip-plugins=emc --tmp-dir " + self.args.remote_dir + " --ticket-number " + str(self.args.ticket_number),
                            code=0, timeout=self.args.timeout)
            except Exception, e:
                error_msg += 'Error running SOS report on:' + str(machine.hostname) + '. Error:' + str(e)
        if error_msg:
            raise Exception(error_msg)



    def Download(self):
        error_msg = ""
        for machine in self.tester.get_component_machines():
            assert isinstance(machine, Machine)
            if machine.distro.name is "vmware":
                continue
            try:
                remote_tarball_path = machine.sys("ls -1 " + self.args.remote_dir + "*" + str(self.args.ticket_number) + "*", code=0)[0]
                tarball = remote_tarball_path.split("/")[-1]
                local_tarball_path = self.args.local_dir + '/' + tarball
                self.tester.debug("Downloading file to: " + local_tarball_path)
                machine.sftp.get(remote_tarball_path, local_tarball_path)
            except Exception, e:
                error_msg += 'Error Downloading from:' + str(machine.hostname) + '. Error:' + str(e)
        if error_msg:
            raise Exception(error_msg)


    def RunAll(self):
        self.Install()
        self.Run()
        self.Download()

if __name__ == "__main__":
    testcase = SOSreport()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["RunAll"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: tony@eucalyptus.com
from platform import machine

from eucaops import Eucaops
import re
import string
from eutester.euinstance import EuInstance
from eutester.eutestcase import EutesterTestCase
from eutester.sshconnection import CommandTimeoutException


class LVMSnapshotUtility(EutesterTestCase):
    
    def __init__(self,extra_args = None):
        self.setuptestcase()
        self.setup_parser()
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.parser.add_argument("--name", default="post_config")
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config, password=self.args.password, download_creds=False)

    def clean_method(self):
        pass

    def CreateLVMSnapshot(self):
        '''
        Create LVM Snapshot
        '''
        machines = self.tester.get_component_machines()
        for machine in machines:
            if machine.distro.name is "vmware":
                continue
            machine.sys("lvcreate -l 100%origin -s -n " + self.args.name + " `blkid -L rootfs`", code=0)

    def RestoreLVMSnapshot(self):
        '''
        Restore LVM Snapshot
        '''

        machines = self.tester.get_component_machines()
        check_file = "/root/merge-executed"
        for machine in machines:
            if machine.distro.name is "vmware":
                continue
            logical_volume = "/dev/vg01/" + self.args.name
            machine.sys("e2label " + logical_volume + " rootfs")
            machine.sys("touch " + check_file)
            machine.sys("lvconvert --merge " + logical_volume, code=0)
            try:
                machine.sys("reboot -f", timeout=2)
            except CommandTimeoutException:
                pass

        self.tester.sleep(30)

        for machine in machines:
            self.tester.ping(machine.hostname, poll_count=120)

        for machine in machines:
            def ssh_refresh():
                try:
                    machine.refresh_ssh()
                    return True
                except:
                    return False
            self.tester.wait_for_result(ssh_refresh, True, timeout=120)
            machine.sys('ls ' + check_file, code=2)
            def lv_gone():
                try:
                    machine.sys("lvdisplay " + logical_volume, code=5)
                    return True
                except:
                    return False
            self.tester.wait_for_result(lv_gone, True, timeout=240)
            machine.sys("lvcreate -l 100%origin -s -n " + logical_volume + " `blkid -L rootfs`", code=0)

    def get_safe_uptime(self, machine):
            uptime = None
            try:
                uptime = machine.get_uptime()
            except: pass
            return uptime


if __name__ == "__main__":
    testcase = LVMSnapshotUtility()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or ["CreateLVMSnapshot"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#!/usr/bin/python
import time
from eutester.eutestcase import EutesterTestCase
from instancetest import InstanceBasics
from eutester.euinstance import EuInstance

class BFEBSBasics(InstanceBasics):
    def __init__(self, name="BFEBSBasics", credpath=None, region=None, config_file=None, password=None, emi=None, zone=None,
                  user_data=None, instance_user=None, imgurl=None ):
        self.imgurl = imgurl
        super(BFEBSBasics, self).__init__(name=name, credpath=credpath, region=region, config_file=config_file, password=password,
                                          emi=emi, zone=zone, user_data=user_data, instance_user=instance_user)

    def clean_method(self):
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
        if self.volume:
            self.tester.delete_volume(self.volume)

    def RegisterImage(self):
        '''Register a BFEBS snapshot'''
        if not self.imgurl:
            raise Exception("No imgurl passed to run BFEBS tests")
        if not self.reservation:
            self.run_instance_params['image'] = self.tester.get_emi(root_device_type="instance-store",
                                                                    not_platform="windows")
            self.reservation = self.tester.run_instance(**self.run_instance_params)
        for instance in self.reservation.instances:
            self.volume = self.tester.create_volume(zone=self.zone, size=3)
            self.volume_device = instance.attach_volume(self.volume)
            instance.sys("curl " +  self.imgurl + " > " + self.volume_device, timeout=800, code=0)
            snapshot = self.tester.create_snapshot(self.volume.id)
            image_id = self.tester.register_snapshot(snapshot)
        self.run_instance_params['image'] = self.tester.get_emi(image_id)
        self.tester.terminate_instances(self.reservation)
        self.reservation = None

    def StopStart(self, zone = None):
        '''Launch a BFEBS instance, stop it then start it again'''
        if zone is None:
            zone = self.zone
        self.RunStop(zone)
        self.StartTerminate(zone)


    def MultipleBFEBSInstances(self):
        """Run half of the available m1.small instances with a BFEBS image"""
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
        self.image = self.tester.get_emi(root_device_type="ebs")
        self.MultipleInstances()

    def ChurnBFEBS(self):
        """Start instances and stop them before they are running, increase time to terminate on each iteration"""
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
        self.image = self.tester.get_emi(root_device_type="ebs")
        self.Churn()

    def RunStop(self, zone=None):
        """Run instance then stop them without starting them again"""
        if zone is None:
            zone = self.zone
        try:
            self.run_instance_params['image'] = self.tester.get_emi(root_device_type="ebs")
        except Exception,e:
            self.RegisterImage()
            self.run_instance_params['image'] = self.tester.get_emi(root_device_type="ebs")
        if not self.volume:
            self.volume = self.tester.create_volume(zone=self.zone, size=2)
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
        self.reservation = self.tester.run_instance(**self.run_instance_params)
        ## Ensure that we can attach and use a volume
        for instance in self.reservation.instances:
            vol_dev = instance.attach_volume(self.volume)
        self.tester.stop_instances(self.reservation)
        for instance in self.reservation.instances:
            if instance.ip_address or instance.private_ip_address:
                raise Exception("Instance had a public " + str(instance.ip_address) + " private " + str(instance.private_ip_address) )
        self.reservation = None

    def StartTerminate(self, zone = None):
        instances = self.tester.get_instances(state="stopped",zone=zone)
        if len(instances) == 0:
            raise Exception("Did not find any stopped instances to start and terminate")
        try:
            for instance in instances:
                self.assertTrue(self.tester.start_instances(instances))
                if self.keypair.name == instance.key_name:
                    instance = self.tester.convert_instance_to_euisntance(instance, keypair=self.keypair)
                    instance.sys("uname -r", code=0)
                else:
                    self.assertTrue(self.tester.ping(instance.ip_address))
        finally:
            self.tester.terminate_instances(instances)
            if self.volume:
                self.tester.wait_for_volume(self.volume, status="available")
                self.tester.delete_volume(self.volume)
                self.volume = None

    def CreateImage(self,zone=None):
        if zone is None:
            zone = self.zone
        try:
            self.run_instance_params['image'] = self.tester.get_emi(root_device_type="ebs")
        except Exception, e:
            self.RegisterImage()
            self.run_instance_params['image'] = self.tester.get_emi(root_device_type="ebs")
        if not self.reservation:
            self.reservation = self.tester.run_instance(**self.run_instance_params)

        ### Run with reboot
        original_image = self.run_instance_params['image']
        for instance in self.reservation.instances:
            assert isinstance(instance, EuInstance)
            self.tester.sleep(60)
            starting_uptime = instance.get_uptime()
            ## Drop a file so we know if we actually created an image
            current_time = str(int(time.time()))
            temp_file = "/root/my-new-file-" + current_time
            instance.sys("touch " + temp_file + " && sync", code=0)
            instance.sys('ls -la', code=0)
            rebooted_image = self.tester.create_image(instance, "BFEBS-test-create-image-reboot-" + current_time)
            instance.connect_to_instance()
            ending_uptime = instance.get_uptime()
            if ending_uptime > starting_uptime:
                raise Exception("Instance did not get stopped then started")
            self.run_instance_params['image'] = rebooted_image
            new_image_reservation = self.tester.run_instance(**self.run_instance_params)
            for new_instance in new_image_reservation.instances:
                ## Check that our temp file exists
                new_instance.sys("ls -la")
                new_instance.sys("ls " + temp_file, code=0)
            self.tester.terminate_instances(new_image_reservation)

        ### Run without reboot
        self.run_instance_params['image'] = original_image
        for instance in self.reservation.instances:
            assert isinstance(instance, EuInstance)
            self.tester.sleep(60)
            starting_uptime = instance.get_uptime()
            ## Drop a file so we know if we actually created an image
            current_time = str(int(time.time()))
            temp_file = "/root/my-new-file-" + current_time
            instance.sys("touch " + temp_file + " && sync", code=0)
            instance.sys('ls -la', code=0)
            not_rebooted_image = self.tester.create_image(instance, "BFEBS-test-create-image-noreboot-" + current_time, no_reboot=True)
            ending_uptime = instance.get_uptime()
            if ending_uptime < starting_uptime:
                raise Exception("Instance did get stopped then started when it shouldn't have")
            self.run_instance_params['image'] = not_rebooted_image
            new_image_reservation = self.tester.run_instance(**self.run_instance_params)
            for new_instance in new_image_reservation.instances:
                ## Check that our temp file exists
                new_instance.sys("ls -la")
                new_instance.sys("ls " + temp_file, code=0)
            self.tester.terminate_instances(new_image_reservation)

if __name__ == "__main__":
    testcase= EutesterTestCase(name='bfebstest')
    testcase.setup_parser(description="Test the Eucalyptus EC2 BFEBS image functionality.")
    testcase.parser.add_argument('--imgurl',
                        help="BFEBS Image to splat down", default=None)
    testcase.get_args()
    bfebstestsuite = testcase.do_with_args(BFEBSBasics)

    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or ["RegisterImage",  "StopStart", "MultipleBFEBSInstances"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = []
    for test in list:
        test = getattr(bfebstestsuite,test)
        unit_list.append(testcase.create_testunit_from_method(test))
    testcase.clean_method = bfebstestsuite.clean_method
    result = testcase.run_test_case_list(unit_list)
    exit(result)


#!/usr/bin/python

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eutester.machine import Machine
from eutester.sshconnection import CommandExitCodeException
import os
import re
import requests
from argparse import ArgumentError
from BeautifulSoup import BeautifulSoup
from logging.handlers import SysLogHandler
from requests import HTTPError
import time
import datetime
import logging
import socket


class ConfigureImagingService(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--img-repo", dest='img_repo', default=None,
                                 help='Repo to install imaging service worker'
                                      'from')
        self.parser.add_argument("--base-url", dest='base_url',
                                 default='http://packages.release.'
                                         'eucalyptus-systems.com/yum/builds/'
                                         'imaging-worker-image/commit/',
                                 help='baseurl used to find most recent commit'
                                      'from')
        self.parser.add_argument("--distro", default=None,
                                 help='Distro name in base_url lookup if used')
        self.parser.add_argument("--worker-keyname", dest='worker_keyname',
                                 default='qa_worker_key',
                                 help='Name used for keypair property. Test '
                                      'will attempt to create the key if it is '
                                      'not found on the system')
        self.parser.add_argument('--ntp-server', dest='ntp_server',
                                 default='0.centos.pool.ntp.org',
                                 help='Worker ntp property string')
        self.parser.add_argument('--zones', default=None,
                                 help='Availability zones imaging service '
                                      'property')
        self.parser.add_argument('--log-server-port', dest='log_server_port',
                                 default=514,
                                 help='log server port for imaging property')
        self.parser.add_argument('--log-server', dest='log_server',
                                 default=None,
                                 help='log server host for imaging property')
        self.parser.add_argument('--task-expiration-hours',
                                 dest='task_expiration_hours',
                                 default=None,
                                 help='task expiration hours for'
                                      ' imaging property')
        self.parser.add_argument('--worker-vmtype', dest='worker_vmtype',
                                 default=None,
                                 help='worker instance type used for'
                                      ' imaging property')

        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops(config_file=self.args.config,
                              password=self.args.password)
        clcs = self.tester.get_component_machines("clc")
        if len(clcs) == 0:
            raise Exception("Unable to find a CLC")
        first_clc = clcs[0]
        assert isinstance(first_clc, Machine)
        self.clc = first_clc
        if self.args.distro is None:
            self.args.distro = self.tester.clc.distro.name
        self.set_repo()




    def clean_method(self):
        pass

    def set_repo(self):
        if self.args.img_repo:
            return
        if not self.args.base_url:
            raise ArgumentError(self.args.img_repo,
                                'Image repo or base_url is required')
        self.args.img_repo = self.get_latest_commit_from_base_url()

    def get_latest_commit_from_base_url(self):
        '''
        Traverse the provided URL, find the latest commit, match the correct
        distro, and release, etc..
        '''
        if not re.search('commit.{0,1}$', self.args.base_url):
            raise ArgumentError('Expected baseurl to end with commit dir')

        #Find latest commit...
        base_url = self.args.base_url
        latest={'timestamp':0, 'count':0, 'commit':None}
        for commit in self._get_dirs_from_body(base_url):
            y, m, d, c = commit.strip('/').split('-')[1:]
            self.debug('commit:{0}, y{1}, m,{2}, d{3}, c{4}'
                       .format(str(commit), y, m, d, c))
            date = "{0}/{1}/{2}".format(d, m, y)
            c = int(c)
            timestamp = time.mktime(
                datetime.datetime.strptime(date, "%d/%m/%Y").timetuple())
            if timestamp >= latest['timestamp']:
                if timestamp == latest['timestamp'] and c <= latest['count']:
                    continue
                latest = {'timestamp':timestamp,
                          'count':c,
                          'commit':commit}
        base_url = base_url + latest['commit']
        #Get the distro
        if (self.args.distro + "/") not in self._get_dirs_from_body(base_url):
            raise ValueError('Distro "' + str(self.args.distro) +
                             '" not found at path:' + str(base_url))
        base_url += self.args.distro + "/"
        #Get the release num
        new_base_url = None
        base_release = self.clc.distro_ver.split('.')[0]
        rel_dirs =  self._get_dirs_from_body(base_url)
        for dir in rel_dirs:
            if re.search('^'+base_release, dir):
                new_base_url = base_url + dir
                break
        if not new_base_url:
            raise RuntimeError(
                'Could find matching release for:' + str(base_release))
        for dir in self._get_dirs_from_body(new_base_url):
            if re.search('x86_64.{0,1}$', dir):
                self.args.base_url = new_base_url + dir
                return self.args.base_url
        raise RuntimeError('Could not find baseurl for image repo from:' +
                            str(self.args.baseurl))

    def _get_dirs_from_body(self, url, retry=3):
        dirs = []
        r = None
        attempt = 0
        while not r and attempt <= retry:
            attempt += 1
            r = requests.get(url)
            try:
                r.raise_for_status()
            except HTTPError as HE:
                self.tester.logger.log.warn('URL:' + str(url) +
                                            ', Attempt:' +
                                            str(attempt) +
                                            "/" + str(retry) +
                                            "HTTP ERROR:" + str(HE))
                if attempt >= retry:
                    raise HE
                time.sleep(1)
        soup = BeautifulSoup(r.text)
        table =soup.find('table')
        rows = table.findAll('tr')
        for row in rows:
            a = row.find('a')
            if a and re.search('/$', a.getText()):
                dirs.append(a.getText())
        self.debug('Paths found at:"' + str(url) + '", paths:' + str(dirs))
        return dirs

    def configure_rsyslog_(self, machine, conf='/etc/rsyslog.conf'):
        '''
        $ModLoad imudp
        $UDPServerRun 514
        $UDPServerAddress 0.0.0.0
        '''
        #Make sure file is at least there
        conf = conf.strip()
        self.debug('checking rsyslog conf file:' + str(conf))
        machine.get_file_stat(conf)
        self._rsyslog_write_value(machine,'$ModLoad imudp',
                                 '\$ModLoad imudp', conf)
        self._rsyslog_write_value(machine, '$UDPServerRun',
                                 '\$UDPServerRun ' +
                                 str(self.args.log_server_port),
                                 conf)
        self._rsyslog_write_value(machine, '$UDPServerAddress',
                                 '\$UDPServerAddress 0.0.0.0', conf)
        machine.sys('service rsyslog restart', code=0)
        machine.sys('iptables -I INPUT -p udp --dport ' +
                    str(self.args.log_server_port) + ' -j ACCEPT', code=0)
        time.sleep(5)
        #Log a message to the server.
        log = logging.getLogger('imaging_setup')
        hdlr = SysLogHandler(address=(machine.hostname, 514),
                             socktype=socket.SOCK_DGRAM)
        log.addHandler(hdlr)
        log.setLevel(logging.DEBUG)
        log.info('EUTESTER Configuring IMAGE SERVICE')


    def _rsyslog_write_value(self, machine, searchkey, new_value, conf):
        try:
            output = machine.sys('cat ' + str(conf) +
                                 " | grep '" + searchkey + "'", code=0)
            for orig_key in output:
                self.debug('Using origkey:' + str(orig_key))
                if not re.search("^#.*." + searchkey, orig_key):
                    machine.sys('perl -p -i -e "s/^*.*' +str(orig_key) + '/\#' +
                            str(orig_key).strip() +
                                '#Commented out by eutester/g" ' + conf,
                            timeout=10, code=0)
        except CommandExitCodeException:
           pass
        machine.sys('echo "' + str(new_value) + '" >> ' + conf, code=0 )


    def configure_service(self):
        """
        This test will attempt to add the repo the remote clc and
        execute the install eucalyptus imaging worker, and configure the
        eucalyptus imaging service.
        """
        self.clc.add_repo(url=self.args.img_repo, name="EucaImagingService")
        self.clc.install("eucalyptus-imaging-worker-image", nogpg=True,
                         timeout=300)
        self.clc.sys("export EUCALYPTUS=" + str(self.tester.eucapath) +
                     " && source " + self.tester.credpath  +
                     "/eucarc && euca-install-imaging-worker --install-default",
                     code=0)
        self.tester.property_manager.show_all_imaging_properties()


    def configure_properties(self):
        #Add the keypair for the imaging service/worker (used for debug)
        try:
            self.keypair = self.tester.get_keypair(self.args.worker_keyname)
        except:
            self.keypair = self.tester.add_keypair(self.args.worker_keyname)
            self.tester.clc.sftp.put(self.args.worker_keyname + '.pem',
                                     self.args.worker_keyname + '.pem')
        key_property = self.tester.property_manager.get_euproperty_by_name(
            'imaging_worker_keyname')
        key_property.set(self.keypair.name)
        #Set the imaging service log server host
        if self.args.log_server:
            log_server = self.args.log_server
            self.logger.log.warn('Not configuring rsyslog on server, assume '
                                 'it is already configured')
        else:
            log_server = self.clc.hostname
            self.debug('Attempting to setup rsyslog on clc...')
            self.configure_rsyslog_(self.clc)
        log_server = self.args.log_server or self.clc.hostname
        log_property = self.tester.property_manager.get_euproperty_by_name(
            'imaging_worker_log_server')
        log_property.set(log_server)
        if self.args.log_server_port:
            log_port_property = \
                self.tester.property_manager.get_euproperty_by_name(
                    'imaging_worker_log_server_port')
            log_port_property.set(self.args.log_server_port)
        #Set the imaging service ntp server
        if self.args.ntp_server:
            ntp_property = self.tester.property_manager.get_euproperty_by_name(
                'imaging_worker_ntp_server')
            ntp_property.set(self.args.ntp_server)
        #Set import task expiration if provided
        if self.args.task_expiration_hours:
            task_property = \
                self.tester.property_manager.get_euproperty_by_name(
                    'import_task_expiration_hours')
            task_property.set(self.args.task_expiration_hours)
        #Set worker instance vm type if provided
        if self.args.worker_vmtype:
            task_property = \
                self.tester.property_manager.get_euproperty_by_name(
                    'imaging_worker_instance_type ')
            task_property.set(self.args.worker_vmtype)
        self.tester.property_manager.show_all_imaging_properties()



if __name__ == "__main__":
    testcase = ConfigureImagingService()
    ### Use the list of tests passed from config/command line to determine
    # what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["configure_service", "configure_properties"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
'''
Create resources (keypairs,groups, volumes,snapshots, buckets) for each user in the cloud. 
'''
from eucaops import Eucaops
import re
import string
from eutester.euinstance import EuInstance
from eutester.eutestcase import EutesterTestCase

class ResourceGeneration(EutesterTestCase):
    
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--no-cleanup", action='store_true')
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( credpath=self.args.credpath, config_file=self.args.config, password=self.args.password)
        self.testers = []

    def clean_method(self):
        if not self.args.no_cleanup:
            for tester in self.testers:
                try:
                    tester.show_euare_whoami()
                except: pass
                tester.cleanup_artifacts()

    def CreateResources(self):
        users = self.tester.get_all_users()
        self.testers.append(self.tester)
        try:
            self.tester.show_all_users()
        except: pass
        for user in users:
            user_name = user['user_name']
            user_account = user['account_name']
            if not re.search("eucalyptus", user_account ):
                self.tester.debug("Creating access key for " + user_name + " in account " + user_account)
                keys = self.tester.create_access_key(user_name=user_name, delegate_account=user_account)
                access_key = keys['access_key_id']
                secret_key = keys['secret_access_key']
                self.tester.debug("Creating Eucaops object with access key " + access_key +
                                  " and secret key " + secret_key)
                new_tester = Eucaops(aws_access_key_id=access_key, aws_secret_access_key=secret_key,
                                     ec2_ip=self.tester.ec2.host, ec2_path=self.tester.ec2.path,
                                     iam_ip=self.tester.euare.host, iam_path=self.tester.euare.path,
                                     s3_ip=self.tester.s3.host, s3_path=self.tester.s3.path,
                                     sts_ip=self.tester.tokens.host, sts_path=self.tester.tokens.path,
                                     cw_ip=self.tester.cw.host, cw_path=self.tester.cw.path,
                                     as_ip=self.tester.autoscale.host, as_path=self.tester.autoscale.path,
                                     elb_ip=self.tester.elb.host, elb_path=self.tester.elb.path,
                                     username=user_name, account=user_account)
                self.testers.append(new_tester)

        self.tester.debug("Created a total of " + str(len(self.testers)) + " testers" )
        try:
            self.tester.show_all_users()
        except: pass
        for resource_tester in self.testers:
            import random
            assert isinstance(resource_tester, Eucaops)
            try:
                resource_tester.show_euare_whoami()
            except:pass
            zone = random.choice(resource_tester.get_zones())
            keypair = resource_tester.add_keypair(resource_tester.id_generator())
            group = resource_tester.add_group(resource_tester.id_generator())
            resource_tester.authorize_group_by_name(group_name=group.name)
            resource_tester.authorize_group_by_name(group_name=group.name, port=-1, protocol="icmp" )
            reservation = resource_tester.run_instance(keypair=keypair.name,group=group.name,zone=zone)
            instance = reservation.instances[0]
            assert isinstance(instance, EuInstance)
            if not instance.ip_address == instance.private_ip_address:
                self.tester.show_all_addresses_verbose()
                address = resource_tester.allocate_address()
                resource_tester.associate_address(instance=instance, address=address)
                resource_tester.disassociate_address_from_instance(instance)
                if not self.args.no_cleanup:
                    resource_tester.release_address(address)
            self.tester.sleep(5)
            instance.update()
            instance.reset_ssh_connection()
            volume = resource_tester.create_volume(size=1, zone=zone)
            instance.attach_volume(volume)
            snapshot = resource_tester.create_snapshot(volume_id=volume.id)
            volume_from_snap = resource_tester.create_volume(snapshot=snapshot, zone=zone)
            bucket = resource_tester.create_bucket(resource_tester.id_generator(12, string.ascii_lowercase  + string.digits))
            key = resource_tester.upload_object(bucket_name=bucket.name, key_name=resource_tester.id_generator(12, string.ascii_lowercase  + string.digits), contents= resource_tester.id_generator(200))
            if not self.args.no_cleanup:
                resource_tester.terminate_instances(reservation)

if __name__ == "__main__":
    testcase = ResourceGeneration()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or [ "CreateResources"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
__author__ = 'clarkmatthew'

import re

class San_Volume_Info():
    def __init__(self, volumeid,  info_dict, san_client):
        self.__dict__ = self.convert_numbers_in_dict(info_dict)
        self.volumeid = volumeid
        self.san_client = san_client
        self.debug = san_client.debug


    def update(self):
        info = self.san_client.get_san_volume_info_by_id(self.volumeid)
        self.__dict__ = self.convert_numbers_in_dict(info)

    def convert_numbers_in_dict(self, dict):
        #convert strings representing numbers to ints
        for key in dict:
            if (re.search("\S", str(dict[key])) and not re.search("\D", str(dict[key]))):
                dict[key] = long(dict[key])
        return dict

    def print_self(self, printmethod=None):
        '''
        formats and prints
        '''
        printmethod = printmethod or self.debug
        buf = "\n"
        longest_key = 0
        for key in self.__dict__:
            if len(key) > longest_key:
                longest_key = len(key)
        for key in self.__dict__:
            buf += str(key).ljust(longest_key) + " -----> :" + str(self.__dict__[key]) + "\n"
        printmethod(buf)



